'''
Brian screensaver

Uses pyscr (google "Python screensaver")
'''
import pyscr


class MySaver(pyscr.Screensaver):
    #set up timer for tick() calls
    TIMEBASE = 1. # called every second

    def configure(self):
        #called to open the screensaver configuration
        #maybe implement here something with venster or win32all
        from ctypes import windll
        windll.user32.MessageBoxA(0,
            "This is the configuration of the Brian screensaver.",
            "Brian Screensaver Configuration", 0, 0, 0)

    def initialize(self):
        #called once when the screensaver is started
        pass

    def finalize(self):
        #called when the screensaver terminates
        pass

    def tick(self):
        #called when the timer tick occurs, set up with startTimer from above
        self.dc.beginDrawing()
        self.dc.setColor(0xffffffL)
        self.dc.setTextColor(0xff0000L)
        self.dc.setBgColor(0x000000L)
        self.dc.setFont("arial")
        #~ self.dc.setBgTransparent(True)
        self.dc.drawText((100, 100), "Brian is thinking...")
        w, h = self.dc.getSize()
        self.dc.drawRect((0, 0), (w - 1, h - 1))
        self.dc.fillEllipse((50, 50), (60, 60))
        self.dc.endDrawing()

#standard 'main' detection and startof screensaver
if __name__ == '__main__':
    pyscr.main()



"""
Monitor AER events with real-time Brian.

I think it works (although not sure) but it's really too slow.
It needs to be vectorised.

Problems with negative timestamps
"""
#import brian_no_units
from brian import *
import socket
from struct import *
from time import time,sleep
from brian.experimental.neuromorphic import *

# Open the UDP socket
UDP_IP="localhost"
UDP_PORT=8991
sock = socket.socket( socket.AF_INET, # Internet
                      socket.SOCK_DGRAM ) # UDP
sock.bind( (UDP_IP,UDP_PORT) )
sock.setblocking(0) # non-blocking

defaultclock.dt=1*ms # so that it's fast enough

R=RealtimeController(dt=100*ms)
   
#retina=SpikeGeneratorGroup(128,[])
allspikes=[]

# Listen to AER
first_event=True
offset=0.
latency=200*ms
@network_operation(EventClock(dt=20*ms))
def listen():
    global first_event,offset,allspikes
    try:
        while True:
            data= sock.recvfrom( 63000 )[0]
            #packet_number=unpack('>I',data[:4]) # not very useful
            #events=[unpack('>II',data[(8*n+4):(8*n+12)]) for n in range(len(data)/8-1)]
            events=fromstring(data[4:],int32).newbyteorder('>')
            if (len(events)>0):
                events=events[:32] # a small selection
                addr=events[::2]
                timestamp=events[1::2]
                if first_event: # Synchronize event timestamps and clock
                    first_event=False
                    t0=time()-start_time
                    ts0=timestamp[0]*1e-6
                    offset=t0-ts0
                _,addr,_=extract_DVS_event(addr)
                #spiketimes=array((addr,timestamp*1e-6+offset+float(latency))).T
                #spiketimes=retina.gather(spiketimes,defaultclock.dt)
                allspikes+=zip(addr,timestamp*1e-6+offset+float(latency))
    except: # no more events (can we do this in a cleaner way?)
        pass

#M=SpikeMonitor(retina)

start_time=time()
run(5*second)

sock.close()

i,t=zip(*allspikes)
plot(t,i,'.')
#raster_plot(M)
show()



from brian import *
from scipy import signal, weave, random
from operator import isSequenceType
from filterbank import Filterbank,RestructureFilterbank
from linearfilterbank import *
from firfilterbank import *
__all__ = ['Cascade',
           'Gammatone',
           'ApproximateGammatone',
           'LogGammachirp',
           'LinearGammachirp',
           'LinearGaborchirp',
           'IIRFilterbank',
           'Butterworth',
           'AsymmetricCompensation',
           'LowPass',
           'asymmetric_compensation_coeffs',
           'BiQuadratic'
           ]


    
class Gammatone(LinearFilterbank):
    '''
    Bank of gammatone filters.
    
    They are implemented as cascades of four 2nd-order IIR filters (this
    8th-order digital filter corresponds to a 4th-order gammatone filter).
    
    The approximated impulse response :math:`\\mathrm{IR}` is defined as follow
    :math:`\\mathrm{IR}(t)=t^3\\exp(-2\\pi b \\mathrm{ERB}(f)t)\\cos(2\\pi f t)`
    where :math:`\\mathrm{ERB}(f)=24.7+0.108 f` [Hz] is the equivalent
    rectangular bandwidth of the filter centered at :math:`f`.

    It comes from Slaney's exact gammatone implementation (Slaney, M., 1993,
    "An Efficient Implementation of the Patterson-Holdsworth 
    Auditory Filter Bank". Apple Computer Technical Report #35). The code is
    based on
    `Slaney's Matlab implementation <http://cobweb.ecn.purdue.edu/~malcolm/interval/1998-010/>`__.
    
    Initialised with arguments:
    
    ``source``
        Source of the filterbank.
        
    ``cf``
        List or array of center frequencies.
        
    ``b=1.019``
        parameter which determines the bandwidth of the filters (and
        reciprocally the duration of its impulse response). In particular, the
        bandwidth = b.ERB(cf), where ERB(cf) is the equivalent bandwidth at
        frequency ``cf``. The default value of ``b`` to a best fit
        (Patterson et al., 1992). ``b`` can either be a scalar and will be the
        same for every channel or an array of the same length as ``cf``.
        
    ``erb_order=1``, ``ear_Q=9.26449``, ``min_bw=24.7``
        Parameters used to compute the ERB bandwidth.
        :math:`\\mathrm{ERB} = ((\mathrm{cf}/\mathrm{ear\\_Q})^{\\mathrm{erb}\\_\\mathrm{order}} + \\mathrm{min\\_bw}^{\\mathrm{erb}\\_\\mathrm{order}})^{(1/\\mathrm{erb}\\_\\mathrm{order})}`.
        Their default values are the ones recommended in
        Glasberg and Moore, 1990. 

    ``cascade=None``
        Specify 1 or 2 to use a cascade of 1 or 2 order 8 or 4 filters instead
        of 4 2nd order filters. Note that this is more efficient but may
        induce numerical stability issues.
    '''

    def __init__(self, source, cf, b=1.019, erb_order=1, ear_Q=9.26449,
                 min_bw=24.7, cascade=None):
        cf = atleast_1d(cf)
        self.cf = cf
        self.samplerate =  source.samplerate
        T = 1/self.samplerate
        self.b,self.erb_order,self.EarQ,self.min_bw=b,erb_order,ear_Q,min_bw
        erb = ((cf/ear_Q)**erb_order + min_bw**erb_order)**(1/erb_order)
        B = b*2*pi*erb
#        B = 2*pi*b

        A0 = T
        A2 = 0
        B0 = 1
        B1 = -2*cos(2*cf*pi*T)/exp(B*T)
        B2 = exp(-2*B*T)
        
        A11 = -(2*T*cos(2*cf*pi*T)/exp(B*T) + 2*sqrt(3+2**1.5)*T*sin(2*cf*pi*T) / \

                exp(B*T))/2
        A12=-(2*T*cos(2*cf*pi*T)/exp(B*T)-2*sqrt(3+2**1.5)*T*sin(2*cf*pi*T)/\
                exp(B*T))/2
        A13=-(2*T*cos(2*cf*pi*T)/exp(B*T)+2*sqrt(3-2**1.5)*T*sin(2*cf*pi*T)/\
                exp(B*T))/2
        A14=-(2*T*cos(2*cf*pi*T)/exp(B*T)-2*sqrt(3-2**1.5)*T*sin(2*cf*pi*T)/\
                exp(B*T))/2

        i=1j
        gain=abs((-2*exp(4*i*cf*pi*T)*T+\
                         2*exp(-(B*T)+2*i*cf*pi*T)*T*\
                                 (cos(2*cf*pi*T)-sqrt(3-2**(3./2))*\
                                  sin(2*cf*pi*T)))*\
                   (-2*exp(4*i*cf*pi*T)*T+\
                     2*exp(-(B*T)+2*i*cf*pi*T)*T*\
                      (cos(2*cf*pi*T)+sqrt(3-2**(3./2))*\
                       sin(2*cf*pi*T)))*\
                   (-2*exp(4*i*cf*pi*T)*T+\
                     2*exp(-(B*T)+2*i*cf*pi*T)*T*\
                      (cos(2*cf*pi*T)-\
                       sqrt(3+2**(3./2))*sin(2*cf*pi*T)))*\
                   (-2*exp(4*i*cf*pi*T)*T+2*exp(-(B*T)+2*i*cf*pi*T)*T*\
                   (cos(2*cf*pi*T)+sqrt(3+2**(3./2))*sin(2*cf*pi*T)))/\
                  (-2/exp(2*B*T)-2*exp(4*i*cf*pi*T)+\
                   2*(1+exp(4*i*cf*pi*T))/exp(B*T))**4)

        allfilts=ones(len(cf))

        self.A0, self.A11, self.A12, self.A13, self.A14, self.A2, self.B0, self.B1, self.B2, self.gain=\
            A0*allfilts, A11, A12, A13, A14, A2*allfilts, B0*allfilts, B1, B2, gain

        self.filt_a=dstack((array([ones(len(cf)), B1, B2]).T,)*4)
        self.filt_b=dstack((array([A0/gain, A11/gain, A2/gain]).T,
                         array([A0*ones(len(cf)), A12, zeros(len(cf))]).T,
                         array([A0*ones(len(cf)), A13, zeros(len(cf))]).T,
                         array([A0*ones(len(cf)), A14, zeros(len(cf))]).T))
    
        LinearFilterbank.__init__(self, source, self.filt_b, self.filt_a)
        if cascade is not None:
            self.decascade(cascade)

class ApproximateGammatone(LinearFilterbank):
    r'''
    Bank of approximate gammatone filters implemented as a cascade of ``order`` IIR gammatone filters.
    
    The filter is derived from the sampled version of the complex analog
    gammatone impulse response
    :math:`g_{\gamma}(t)=t^{\gamma-1} (\lambda e^{i \eta t})^{\gamma}` 
    where :math:`\gamma` corresponds to ``order``, :math:`\eta` defines the
    oscillation frequency ``cf``, and :math:`\lambda` defines the bandwidth
    parameter.
    
    The design is based on the Hohmann implementation as described in
    Hohmann, V., 2002, "Frequency analysis and synthesis using a Gammatone
    filterbank", Acta Acustica United with Acustica. The code is based on the
    Matlab gammatone implementation from
    `Meddis' toolbox <http://www.essex.ac.uk/psychology/psy/PEOPLE/meddis/webFolder08/WebIntro.htm>`__. 
    
    Initialised with arguments:
    
    ``source``
        Source of the filterbank.
        
    ``cf``
        List or array of center frequencies.
        
    ``bandwidth``
        List or array of filters bandwidth corresponding, one for each cf.
        
    ``order=4``
        The number of 1st-order gammatone filters put in cascade, and therefore
        the order the resulting gammatone filters.
     '''
   
    def __init__(self, source, cf,  bandwidth,order=4):
        cf = atleast_1d(cf)
        bandwidth = atleast_1d(bandwidth)
        self.cf = cf
        self.samplerate =  source.samplerate
        dt = 1/self.samplerate 
        phi = 2 * pi * bandwidth * dt
        theta = 2 * pi * cf * dt
        cos_theta = cos(theta)
        sin_theta = sin(theta)
        alpha = -exp(-phi) * cos_theta
        b0 = ones(len(cf))
        b1 = 2 * alpha
        b2 = exp(-2 * phi)
        z1 = (1 + alpha * cos_theta) - (alpha * sin_theta) * 1j
        z2 = (1 + b1 * cos_theta) - (b1 * sin_theta) * 1j
        z3 = (b2 * cos(2 * theta)) - (b2 * sin(2 * theta)) * 1j
        tf = (z2 + z3) / z1
        a0 = abs(tf)
        a1 = alpha * a0   
        # we apply the same filters order times so we just duplicate them in the 3rd axis for the parallel_lfilter_step command
        self.filt_a = dstack((array([b0, b1, b2]).T,)*order)
        self.filt_b = dstack((array([a0, a1, zeros(len(cf))]).T,)*order)
        self.order = order
        
        LinearFilterbank.__init__(self,source, self.filt_b, self.filt_a)

           
class LogGammachirp(LinearFilterbank):
    r'''
    Bank of gammachirp filters with a logarithmic frequency sweep.
    
    The approximated impulse response :math:`\mathrm{IR}` is defined as follows:
    :math:`\mathrm{IR}(t)=t^3e^{-2\pi b \mathrm{ERB}(f)t}\cos(2\pi (f t +c\cdot\ln(t))`
    where :math:`\mathrm{ERB}(f)=24.7+0.108 f` [Hz] is the equivalent
    rectangular bandwidth of the filter centered at :math:`f`.
    
    The implementation is a cascade of 4 2nd-order IIR gammatone filters 
    followed by a cascade of ncascades 2nd-order asymmetric compensation filters
    as introduced in Unoki et al. 2001, "Improvement of an IIR asymmetric 
    compensation gammachirp filter". 

    Initialisation parameters:
    
    ``source``
        Source sound or filterbank.
        
    ``f``
        List or array of the sweep ending frequencies
        (:math:`f_{\mathrm{instantaneous}}=f+c/t`). 
        
    ``b=1.019``
        Parameters which determine the duration of the impulse response.
        ``b`` can either be a scalar and will be the same for every channel or
        an array with the same length as ``f``.
        
    ``c=1``
        The glide slope (or sweep rate) given in Hz/second. The trajectory of
        the instantaneous frequency towards f is an upchirp when c<0 and a 
        downchirp when c>0.
        ``c`` can either be a scalar and will be the same for every channel or
        an array with the same length as ``f``.
        
    ``ncascades=4``
        Number of times the asymmetric compensation filter is put in cascade.
        The default value comes from Unoki et al. 2001. 
    '''
      
    def __init__(self, source, f,b=1.019,c=1,ncascades=4):
        f = atleast_1d(f)
        self.f = f
        self.samplerate= source.samplerate
        
        self.c=c
        self.b=b
        gammatone=Gammatone(source, f,b)

        self.gammatone_filt_b=gammatone.filt_b
        self.gammatone_filt_a=gammatone.filt_a

        ERBw=24.7*(4.37e-3*f+1.)

        p0=2
        p1=1.7818*(1-0.0791*b)*(1-0.1655*abs(c))
        p2=0.5689*(1-0.1620*b)*(1-0.0857*abs(c))
        p3=0.2523*(1-0.0244*b)*(1+0.0574*abs(c))
        p4=1.0724

        self.asymmetric_filt_b=zeros((len(f),3, ncascades))
        self.asymmetric_filt_a=zeros((len(f),3, ncascades))

        self.asymmetric_filt_b,self.asymmetric_filt_a=asymmetric_compensation_coeffs(self.samplerate,f,self.asymmetric_filt_b,self.asymmetric_filt_a,b,c,p0,p1,p2,p3,p4)

        #concatenate the gammatone filter coefficients so that everything is in cascade in each frequency channel
        self.filt_b=concatenate([self.gammatone_filt_b, self.asymmetric_filt_b],axis=2)
        self.filt_a=concatenate([self.gammatone_filt_a, self.asymmetric_filt_a],axis=2)
        
        LinearFilterbank.__init__(self, source, self.filt_b,self.filt_a)


class LinearGammachirp(FIRFilterbank):
    r'''
    Bank of gammachirp filters with linear frequency sweeps and gamma envelope
    as described in Wagner et al. 2009, "Auditory responses in the barn owl's
    nucleus laminaris to clicks: impulse response and signal analysis of
    neurophonic potential", J. Neurophysiol.
    
    The impulse response :math:`\mathrm{IR}` is defined as follow
    :math:`\mathrm{IR}(t)=t^3e^{-t/\sigma}\cos(2\pi (f t +c/2 t^2)+\phi)`
    where :math:`\sigma` corresponds to ``time_constant`` and :math:`\phi` to
    ``phase`` (see definition of parameters).

    Those filters are implemented as FIR filters using truncated time
    representations of gammachirp functions as the impulse response. The impulse
    responses, which need to have the same length for every channel, have a
    duration of 15 times the biggest time constant. The length of the impulse
    response is therefore ``15*max(time_constant)*sampling_rate``.  The impulse
    responses are normalized with respect to the transmitted power, i.e.
    the rms of the filter taps is 1.
    
    Initialisation parameters:
    
    ``source``
        Source sound or filterbank.
        
    ``f``
        List or array of the sweep starting frequencies
        (:math:`f_{\mathrm{instantaneous}}=f+ct`).
        
    ``time_constant``
        Determines the duration of the envelope and consequently the length of
        the impulse response.
        
    ``c=1``
        The glide slope (or sweep rate) given in Hz/second. The time-dependent
        instantaneous frequency is ``f+c*t`` and is therefore going upward when
        c>0 and downward when c<0. ``c`` can either be a scalar and will be the
        same for every channel or an array with the same length as ``f``.
        
    ``phase=0``
        Phase shift of the carrier.
        
    Has attributes:
    
    ``length_impulse_response`` 
        Number of samples in the impulse responses.
        
    ``impulse_response``
        Array of shape ``(nchannels, length_impulse_response)`` with each row
        being an impulse response for the corresponding channel.
    '''
    def __init__(self,source, f, time_constant, c, phase=0): 
        
        self.f=f=atleast_1d(f)
        self.c=c=atleast_1d(c)
        self.phase=phase=atleast_1d(phase)
        self.time_constant=time_constant=atleast_1d(time_constant) 
        if len(time_constant)==1:
            time_constant=time_constant*ones(len(f))
        if len(c)==1:
            c=c*ones(len(f))
        if len(phase)==1:
            phase=phase*ones(len(f))
        self.samplerate= source.samplerate
        
        
        
        Tcst_max=max(time_constant)

        t_start=-Tcst_max*3*second
        t=arange(t_start,-4*t_start,1./self.samplerate)

        
            
        self.impulse_response=zeros((len(f),len(t)))
                                    
        for ich in xrange(len(f)):
            env=(t-t_start)**3*exp(-(t-t_start)/time_constant[ich])        
            self.impulse_response[ich,:]=env*cos(2*pi*(f[ich]*t+c[ich]/2*t**2)+phase[ich])
#            self.impulse_response[ich,:]=self.impulse_response[ich,:]/sqrt(sum(self.impulse_response[ich,:]**2))
            self.impulse_response[ich,:]=self.impulse_response[ich,:]/sum(abs(self.impulse_response[ich,:]))        


        FIRFilterbank.__init__(self,source, self.impulse_response)

class LinearGaborchirp(FIRFilterbank):
    r'''
    Bank of gammachirp filters with linear frequency sweeps and gaussian envelope
    as described in Wagner et al. 2009, "Auditory responses in the barn owl's 
    nucleus laminaris to clicks: impulse response and signal analysis of
    neurophonic potential", J. Neurophysiol.
    
    The impulse response :math:`\mathrm{IR}` is defined as follows:
    :math:`\mathrm{IR}(t)=e^{-t/2\sigma^2}\cos(2\pi (f t +c/2 t^2)+\phi)`,
    where :math:`\sigma` corresponds to ``time_constant`` and :math:`\phi` to
    ``phase`` (see definition of parameters).
    
    These filters are implemented as FIR filters using truncated time
    representations of gammachirp functions as the impulse response. The impulse
    responses, which need to have the same length for every channel, have a
    duration of 12 times the biggest time constant. The length of the impulse
    response is therefore ``12*max(time_constant)*sampling_rate``. The envelope
    is a gaussian function (Gabor filter).  The impulse responses are normalized
    with respect to the transmitted  power, i.e. the rms of the filter taps is
    1.
    
    Initialisation parameters:
    
    ``source``
        Source sound or filterbank.
        
    ``f``
        List or array of the sweep starting frequencies
        (:math:`f_{\mathrm{instantaneous}}=f+c*t`).
        
    ``time_constant``
        Determines the duration of the envelope and consequently the length of
        the impluse response.
        
    ``c=1``
        The glide slope (or sweep rate) given ins Hz/second. The time-dependent
        instantaneous frequency is ``f+c*t`` and is therefore going upward when
        c>0 and downward when c<0. ``c`` can either be a scalar and will be the
        same for every channel or an array with the same length as ``f``.
        
    ``phase=0``
        Phase shift of the carrier.
        
    Has attributes:
    
    ``length_impulse_response`` 
        Number of sample in the impulse responses.
        
    ``impulse_response``
        Array of shape ``(nchannels, length_impulse_response)`` with each row
        being an impulse response for the corresponding channel.
    '''
    def __init__(self,source, f, time_constant, c, phase=0): 
        self.f=f=atleast_1d(f)
        self.c=c=atleast_1d(c)
        self.phase=phase=atleast_1d(phase)
        self.time_constant=time_constant=atleast_1d(time_constant) 
        if len(time_constant)==1:
            time_constant=time_constant*ones(len(f))
        if len(c)==1:
            c=c*ones(len(f))
        if len(phase)==1:
            phase=phase*ones(len(f))
        self.samplerate = source.samplerate
        
        Tcst_max=max(time_constant)

        t_start=-Tcst*6*second
        t=arange(t_start,-t_start,1./self.samplerate)

        self.impulse_response=zeros((len(f),len(t)))
                                    
        for ich in xrange(len(f)):
            env=exp(-(t/(2*time_constant[ich]))**2)   
            self.impulse_response[ich,:]=env*cos(2*pi*(f[ich]*t+c[ich]/2*t**2)+phase[ich])
            self.impulse_response[ich,:]=self.impulse_response[ich,:]/sqrt(sum(self.impulse_response[ich,:]**2))

        FIRFilterbank.__init__(self, source, self.impulse_response)

class IIRFilterbank(LinearFilterbank):
    '''
    Filterbank of IIR filters. The filters can be low, high, bandstop or
    bandpass and be of type Elliptic, Butterworth, Chebyshev etc. The
    ``passband``  and ``stopband`` can be scalars (for low or high pass) or
    pairs of parameters (for stopband and passband) yielding similar filters for
    every channel. They can also be arrays of shape ``(1, nchannels)`` for low
    and high pass or ``(2, nchannels)`` for stopband and passband yielding
    different filters along channels. This class uses the scipy iirdesign
    function to generate filter coefficients for every channel. 
    
    See the documentation for scipy.signal.iirdesign for more details.
    
    Initialisation parameters:
    
    ``samplerate``
        The sample rate in Hz.
        
    ``nchannels``
        The number of channels in the bank
        
    ``passband``, ``stopband``
        The edges of the pass and stop bands in Hz. For lowpass and highpass
        filters, in the case of similar filters for each channel, they are
        scalars and passband<stopband for low pass or stopband>passband for a
        highpass. For a bandpass or bandstop filter, in the case of similar
        filters for each channel, make passband and stopband a list with two
        elements, e.g. for a bandpass have ``passband=[200*Hz, 500*Hz]`` and
        ``stopband=[100*Hz, 600*Hz]``. ``passband`` and ``stopband`` can also be
        arrays of shape ``(1, nchannels)`` for low and high pass or
        ``(2, nchannels)`` for stopband and passband yielding different filters
        along channels.
        
    ``gpass``
        The maximum loss in the passband in dB. Can be a scalar or an array of
        length ``nchannels``.
        
    ``gstop``
        The minimum attenuation in the stopband in dB. Can be a scalar or an
        array of length ``nchannels``.
        
    ``btype``
        One of 'low', 'high', 'bandpass' or 'bandstop'.
    
    ``ftype``
        The type of IIR filter to design:
        'ellip' (elliptic),
        'butter' (Butterworth),
        'cheby1' (Chebyshev I),
        'cheby2' (Chebyshev II),
        'bessel' (Bessel).
    
    '''
    
    def __init__(self, source, nchannels, passband, stopband, gpass, gstop, btype, ftype):

        Wpassband = passband.copy()
        Wstopband = stopband.copy()
        Wpassband = atleast_1d(Wpassband)
        Wstopband = atleast_1d(Wstopband)
        gpass = atleast_1d(gpass)
        gstop = atleast_1d(gstop)
        
        self.samplerate=source.samplerate
        if Wpassband.shape != Wstopband.shape:
            raise Exeption('passband and stopband must contain the same number of ent')
        try:
            Wpassband=Wpassband/self.samplerate*2+0.0    # wn=1 corresponding to half the sample rate 
            Wstopband=Wstopband/self.samplerate*2+0.0     
        except DimensionMismatchError:
            raise DimensionMismatchError('IIRFilterbank passband, stopband parameters must be in Hz')
        
        # now design filterbank      
        if btype=='low' or btype=='high':
            if len(Wpassband)==1:     #if there is only one Wn value for all channel just repeat it
                self.filt_b, self.filt_a = signal.iirdesign(Wpassband, Wstopband, gpass, gstop, ftype=ftype)
                self.filt_b=kron(ones((nchannels,1)),self.filt_b)
                self.filt_a=kron(ones((nchannels,1)),self.filt_a)
            else:               #else make nchannels different filters
                if len(gstop) != nchannels: #if the ripple parameters are scalar make them as long as the number of channels
                    gpass=repeat(gpass,nchannels)
                if len(gstop) != nchannels:
                    gstop=repeat(gstop,nchannels)
                order=0
                filt_b, filt_a =[1]*nchannels,[1]*nchannels  
                for i in xrange((nchannels)): #generate the different filter coeffcients
                    filt_b[i], filt_a[i] = signal.iirdesign(Wpassband[i], Wstopband[i], gpass[i], gstop[i], ftype=ftype)
                if len(filt_b[i])>order: #take the highst order of them to be the size of the filter coefficient matrix
                    order=len(filt_b[i])
                self.filt_b=zeros((nchannels,order))
                self.filt_a=zeros((nchannels,order))
                for i in xrange((nchannels)): #fill the coefficient matrix 
                    self.filt_b[i,:len(filt_b[i])], self.filt_a[i,:len(filt_a[i])] = filt_b[i],filt_a[i]
        else:
            if Wpassband.ndim==1:     #if there is only one Wn pair of values for all channel just repeat it
                self.filt_b, self.filt_a = signal.iirdesign(Wpassband, Wstopband, gpass, gstop, ftype=ftype)
                self.filt_b=kron(ones((nchannels,1)),self.filt_b)
                self.filt_a=kron(ones((nchannels,1)),self.filt_a)
            else:   
                if len(gstop) != nchannels:#if the ripple parameters are scalar make them as long as the number of channels
                    gpass=repeat(gpass,nchannels)
                if len(gstop) != nchannels:
                    gstop=repeat(gstop,nchannels)
                order=0
                filt_b, filt_a =[1]*nchannels,[1]*nchannels
                for i in xrange((nchannels)):#take the highst order of them to be the size of the filter coefficient matrix
                    filt_b[i], filt_a[i] = signal.iirdesign(Wpassband[:,i], Wstopband[:,i], gpass[i], gstop[i], ftype=ftype)
                if len(filt_b[i])>order:
                    order=len(filt_b[i])
                self.filt_b=zeros((nchannels,order))
                self.filt_a=zeros((nchannels,order))
                for i in xrange((nchannels)):#fill the coefficient matrix 
                    self.filt_b[i,:len(filt_b[i])], self.filt_a[i,:len(filt_a[i])] = filt_b[i],filt_a[i]
                    
        
        self.filt_a=self.filt_a.reshape(self.filt_a.shape[0],self.filt_a.shape[1],1)
        self.filt_b=self.filt_b.reshape(self.filt_b.shape[0],self.filt_b.shape[1],1)  
        self.nchannels = nchannels
        self.passband = passband
        self.stopband = stopband
        self.gpass = gpass
        self.gstop = gstop
        self.ftype= ftype
        self.order= self.filt_a.shape[1]-1
        LinearFilterbank.__init__(self,source, self.filt_b, self.filt_a) 


class Butterworth(LinearFilterbank):
    '''
    Filterbank of  low, high, bandstop or bandpass  Butterworth filters. 
    The cut-off frequencies or the band frequencies can either be the same for
    each channel or different along channels.
    
    Initialisation parameters:
    
    ``samplerate``
        Sample rate.
        
    ``nchannels``
        Number of filters in the bank.
        
    ``order``
        Order of the filters.
        
    ``fc``
        Cutoff parameter(s) in Hz. For the case of a lowpass or highpass
        filterbank, ``fc`` is either a scalar (thus the same value for all of
        the channels) or an array  of length ``nchannels``. For the case of a
        bandpass or bandstop, ``fc`` is either a pair of scalar defining the
        bandpass or bandstop (thus the same values for all of the channels) or
        an array of shape ``(2, nchannels)`` to define a pair for every channel.
        
    ``btype``
        One of 'low', 'high', 'bandpass' or 'bandstop'.
    '''

    def __init__(self,source, nchannels, order, fc, btype='low'):
        Wn=fc.copy()
        Wn=atleast_1d(Wn) #Scalar inputs are converted to 1-dimensional arrays
        self.samplerate = source.samplerate
        try:
            Wn= Wn/self.samplerate *2+0.0    # wn=1 corresponding to half the sample rate   
        except DimensionMismatchError:
            raise DimensionMismatchError('Wn must be in Hz')
        

        
        if btype=='low' or btype=='high':
            self.filt_b=zeros((nchannels,order+1))
            self.filt_a=zeros((nchannels,order+1))
            if len(Wn)==1:     #if there is only one Wn value for all channel just repeat it
                self.filt_b, self.filt_a = signal.butter(order, Wn, btype=btype)
                self.filt_b=kron(ones((nchannels,1)),self.filt_b)
                self.filt_a=kron(ones((nchannels,1)),self.filt_a)
            else:               #else make nchannels different filters
                for i in xrange((nchannels)):
                    self.filt_b[i,:], self.filt_a[i,:] = signal.butter(order, Wn[i], btype=btype)
        else:
            self.filt_b=zeros((nchannels,2*order+1))
            self.filt_a=zeros((nchannels,2*order+1))
            if Wn.ndim==1:     #if there is only one Wn pair of values for all channel just repeat it
                self.filt_b, self.filt_a = signal.butter(order, Wn, btype=btype)
                self.filt_b=kron(ones((nchannels,1)),self.filt_b)
                self.filt_a=kron(ones((nchannels,1)),self.filt_a)
            else:   
                for i in xrange((nchannels)):
                    self.filt_b[i,:], self.filt_a[i,:] = signal.butter(order, Wn[:,i], btype=btype)   
                    
                    
        self.filt_a=self.filt_a.reshape(self.filt_a.shape[0],self.filt_a.shape[1],1)
        self.filt_b=self.filt_b.reshape(self.filt_b.shape[0],self.filt_b.shape[1],1)  
        self.nchannels = nchannels    
        LinearFilterbank.__init__(self,source, self.filt_b, self.filt_a) 


class BiQuadratic(LinearFilterbank):
    '''
    Bank of biquadratic bandpass filters
    
    The transfer function of the filters are like the ones of  all second-order linear filters
    :math:`H(s)=\frac{Kw_{0}^{2}}{s_{2}+w_{0}/Qs+w_{0}^{2}}`
    where :math:`w_{0}`  is the centre frequency and :math:`Q` the quality factor of the filter

    
    The implementation  is a 2nd-order IIR  filter with a tranfer function being the ratio of two quadratic functions.
    

    Initialisation parameters:
    
    ``source``
        Source sound or filterbank.
        
    ``f``
        List or array of the centre frequencies. (:math:`w_{0}^{2}/2\pi`) 
        
        
    ``Q``
        Quality factor of the filters (dimensionless). It can be a scalar (the same for every channel) or a list/array.``Q`` defines the bandwidth such that
        
        
    ``BW``
        Alternativl
        
    '''
      
    def __init__(self, source, cf,Q):
        cf=cf[0]
        Q=Q[0]
#        Q=1
#        cf = atleast_1d(cf)
        self.samplerate= source.samplerate
        w0 = 2*pi*cf/self.samplerate/second
        BW = 2./log(2)*arcsinh((1./2/Q))
        alpha = sin(w0)*sinh(log(2)/2 * BW * w0/sin(w0) )
        
        
        
        b_temp =   array([alpha,0,-alpha])/(1 + alpha)
        a_temp = array([1 + alpha,-2*cos(w0),1 - alpha])/(1 + alpha)
        
        print  b_temp,a_temp
        self.filt_b = tile(b_temp.reshape([3,1]),[1,1,1])               
        self.filt_a = tile(a_temp.reshape([3,1]),[1,1,1]) 
        
        LinearFilterbank.__init__(self, source, self.filt_b,self.filt_a)

class LowPass(LinearFilterbank):
    '''
    Bank of 1st-order lowpass filters
    
    The code is based on the code found in the
    `Meddis toolbox <http://www.essex.ac.uk/psychology/psy/PEOPLE/meddis/webFolder08/WebIntro.htm>`__. 
    It was implemented here to be used in the DRNL cochlear model implementation.

    Initialised with arguments:
    
    ``source``
        Source of the filterbank.
        
    ``fc``
        Value, list or array (with length = number of channels) of cutoff
        frequencies.
    '''
    def __init__(self,source,fc):
        if not isSequenceType(fc):
            fc = fc*ones(source.nchannels)
        nchannels=len(fc)
        self.samplerate= source.samplerate
        dt=1./self.samplerate

        self.filt_b=zeros((nchannels, 2, 1))
        self.filt_a=zeros((nchannels, 2, 1))
        tau=1/(2*pi*fc)
        self.filt_b[:,0,0]=dt/tau
        self.filt_b[:,1,0]=0*ones(nchannels)
        self.filt_a[:,0,0]=1*ones(nchannels)
        self.filt_a[:,1,0]=-(1-dt/tau)
        LinearFilterbank.__init__(self,source, self.filt_b, self.filt_a) 

      
class Cascade(LinearFilterbank):
    '''
    Cascade of ``n`` times a linear filterbank. 
    
    Initialised with arguments:
    
    ``source``
        Source of the new filterbank.
        
    ``filterbank``
        Filterbank object to be put in cascade
        
    ``n``
        Number of cascades
    '''
    
    def __init__(self,source, filterbank,n):
        b=filterbank.filt_b
        a=filterbank.filt_a
        self.samplerate =  source.samplerate
        self.nchannels=filterbank.nchannels
        self.filt_b=zeros((b.shape[0], b.shape[1],n))
        self.filt_a=zeros((a.shape[0], a.shape[1],n))
        for i in range((n)):
            self.filt_b[:,:,i]=b[:,:,0]
            self.filt_a[:,:,i]=a[:,:,0]
            
        LinearFilterbank.__init__(self, source,self.filt_b, self.filt_a)  
        
class AsymmetricCompensation(LinearFilterbank):
    '''
    Bank of asymmetric compensation filters.
    
    Those filters are meant to be used in cascade with gammatone filters to
    approximate gammachirp filters (Unoki et al., 2001, Improvement of
    an IIR asymmetric compensation gammachirp filter, Acoust. Sci. & Tech.).
    They are implemented a a cascade of low order filters. The code 
    is based on the implementation found in the
    `AIM-MAT toolbox <http://www.pdn.cam.ac.uk/groups/cnbh/aimmanual/index.html>`__.

    Initialised with arguments:
    
    ``source``
        Source of the filterbank.
        
    ``f``
        List or array of the cut off frequencies.
        
    ``b=1.019``
        Determines the duration of the impulse response.
        Can either be a scalar and will be the same for every channel or
        an array with the same length as ``cf``.
        
    ``c=1``
        The glide slope when this filter is used to implement a gammachirp.
        Can either be a scalar and will be the same for every channel or
        an array with the same length as ``cf``.
        
    ``ncascades=4``
        The number of time the basic filter is put in cascade.
     '''
     
    def __init__(self, source, f,b=1.019, c=1,ncascades=4):
        
        f = atleast_1d(f)
        self.f = f
        self.samplerate =  source.samplerate     
        ERBw=24.7*(4.37e-3*f+1.)
        p0=2
        p1=1.7818*(1-0.0791*b)*(1-0.1655*abs(c))
        p2=0.5689*(1-0.1620*b)*(1-0.0857*abs(c))
        p3=0.2523*(1-0.0244*b)*(1+0.0574*abs(c))
        p4=1.0724

        self.filt_b=zeros((len(f), 3, ncascades))
        self.filt_a=zeros((len(f), 3, ncascades))

        for k in arange(ncascades):

            r=exp(-p1*(p0/p4)**(k)*2*pi*b*ERBw/self.samplerate) #k instead of k-1 because range 0 N-1
            Df=(p0*p4)**(k)*p2*c*b*ERBw

            phi=2*pi*maximum((f+Df), 0)/self.samplerate
            psy=2*pi*maximum((f-Df), 0)/self.samplerate

            ap=vstack((ones(r.shape),-2*r*cos(phi), r**2)).T
            bz=vstack((ones(r.shape),-2*r*cos(psy), r**2)).T

            fn=f#+ compensation_filter_order* p3 *c *b *ERBw/4;

            vwr=exp(1j*2*pi*fn/self.samplerate)
            vwrs=vstack((ones(vwr.shape), vwr, vwr**2)).T

            ##normilization stuff
            nrm=abs(sum(vwrs*ap, 1)/sum(vwrs*bz, 1))
            
            bz=bz*tile(nrm,[3,1]).T
            self.filt_b[:, :, k]=bz
            self.filt_a[:, :, k]=ap

        LinearFilterbank.__init__(self, source, self.filt_b, self.filt_a)



def asymmetric_compensation_coeffs(samplerate,fr,filt_b,filt_a,b,c,p0,p1,p2,p3,p4):
    '''
    This function is used to generated the coefficient of the asymmetric
    compensation filter used for the gammachirp implementation.
    '''
    ERBw=24.7*(4.37e-3*fr+1.)
    nbr_cascade=4
    for k in arange(nbr_cascade):

        r=exp(-p1*(p0/p4)**(k)*2*pi*b*ERBw/samplerate) #k instead of k-1 because range 0 N-1

        Dfr=(p0*p4)**(k)*p2*c*b*ERBw

        phi=2*pi*maximum((fr+Dfr), 0)/samplerate
        psy=2*pi*maximum((fr-Dfr), 0)/samplerate

        ap=vstack((ones(r.shape),-2*r*cos(phi), r**2)).T
        bz=vstack((ones(r.shape),-2*r*cos(psy), r**2)).T

        vwr=exp(1j*2*pi*fr/samplerate)
        vwrs=vstack((ones(vwr.shape), vwr, vwr**2)).T

        ##normilization stuff
        nrm=abs(sum(vwrs*ap, 1)/sum(vwrs*bz, 1))
        bz=bz*tile(nrm,[3,1]).T
        filt_b[:, :, k]=bz
        filt_a[:, :, k]=ap

    return filt_b,filt_a

def factorial(n):
    return prod(arange(1, n+1))



'''
The first step in making a new release of Brian is to run all the
tests to make sure that nothing has been broken by any new
changes, and that every object is still behaving as we say it
should.
'''

from brian import *
run_all_tests()



"""
Mirollo-Strogatz network
"""
from brian import *

tau = 10 * ms
v0 = 11 * mV
N = 20
w = .1 * mV

group = NeuronGroup(N, model='dv/dt=(v0-v)/tau : volt', threshold=10 * mV, reset=0 * mV)

W = Connection(group, group, 'v', weight=w)

group.v = rand(N) * 10 * mV

S = SpikeMonitor(group)

run(300 * ms)

raster_plot(S)
show()



from brian import *
set_global_preferences(usenewbrianhears=True,
                       useweave=False)
from brian.hears import *

x = Sound(randn(1000, 10), samplerate=44100*Hz)

class AccumulateFilterbank(Filterbank):
    def __init__(self, source):
        Filterbank.__init__(self, source)
        self.nchannels = 1
    def buffer_apply(self, input):
        return reshape(sum(input, axis=1), (input.shape[0], 1))

fb = AccumulateFilterbank(x)

fb.buffer_init()

y = fb.buffer_fetch(0, 1000)

subplot(211)
plot(x)
subplot(212)
plot(y)
plot(sum(x, axis=1))
show()



from numpy import *
if __name__ == '__main__':
    from brian import *
    from brian.experimental.new_c_propagate import *
    from brian.experimental.codegen.expressions import *
    from brian.inspection import modified_variables, namespace
    from brian.optimiser import freeze
    from brian.utils.separate_equations import separate_equations
    from brian.experimental.codegen.c_support_code import *
else:
    from ..utils.documentation import flattened_docstring
    from ..stdunits import ms
    from ..connections import Connection, DelayConnection
    from ..log import log_debug, log_warn
    from ..monitor import RecentStateMonitor, SpikeMonitor
    from ..network import NetworkOperation
    from ..neurongroup import NeuronGroup
    from ..equations import Equations
    from new_c_propagate import *
    from ..inspection import modified_variables, namespace
    from codegen.expressions import *
    from ..optimiser import freeze
    from ..utils.separate_equations import separate_equations
    from codegen.c_support_code import *
from scipy import weave
import re

__all__ = ['CSTDP']


class CSTDP(NetworkOperation):
    def __init__(self, C, eqs, pre, post, wmin=0, wmax=Inf, level=0,
                 clock=None, delay_pre=None, delay_post=None):
        NetworkOperation.__init__(self, lambda:None, clock=clock)
        C.compress()
        # Convert to equations object
        if isinstance(eqs, Equations):
            eqs_obj = eqs
        else:
            eqs_obj = Equations(eqs, level=level + 1)
        # handle multi-line pre, post equations and multi-statement equations separated by ;
        if '\n' in pre:
            pre = flattened_docstring(pre)
        elif ';' in pre:
            pre = '\n'.join([line.strip() for line in pre.split(';')])
        if '\n' in post:
            post = flattened_docstring(post)
        elif ';' in post:
            post = '\n'.join([line.strip() for line in post.split(';')])

        # Check units
        eqs_obj.compile_functions()
        eqs_obj.check_units()

        # Get variable names
        vars = eqs_obj._diffeq_names
        # Find which ones are directly modified (e.g. regular expression matching; careful with comments)
        vars_pre = [var for var in vars if var in modified_variables(pre)]
        vars_post = [var for var in vars if var in modified_variables(post)]

        # additional dependencies are used to ensure that if there are multiple
        # pre/post separated equations they are grouped together as one
        additional_deps = ['__pre_deps='+'+'.join(vars_pre),
                           '__post_deps='+'+'.join(vars_post)]
        separated_equations = separate_equations(eqs_obj, additional_deps)
        if not len(separated_equations) == 2:
            print separated_equations
            raise ValueError('Equations should separate into pre and postsynaptic variables.')
        sep_pre, sep_post = separated_equations
        for v in vars_pre:
            if v in sep_post._diffeq_names:
                sep_pre, sep_post = sep_post, sep_pre
                break

        index_pre = [i for i in range(len(vars)) if vars[i] in vars_pre or vars[i] in sep_pre._diffeq_names]
        index_post = [i for i in range(len(vars)) if vars[i] in vars_post or vars[i] in sep_post._diffeq_names]

        vars_pre = array(vars)[index_pre].tolist()
        vars_post = array(vars)[index_post].tolist()

        # Check pre/post consistency
        shared_vars = set(vars_pre).intersection(vars_post)
        if shared_vars != set([]):
            raise Exception, str(list(shared_vars)) + " are both presynaptic and postsynaptic!"

        # Substitute equations/aliases into pre/post code
        def substitute_eqs(code):
            for name in sep_pre._eq_names[-1::-1]+sep_post._eq_names[-1::-1]: # reverse order, as in equations.py
                if name in sep_pre._eq_names:
                    expr = sep_pre._string[name]
                else:
                    expr = sep_post._string[name]
                code = re.sub("\\b" + name + "\\b", '(' + expr + ')', code)
            return code
        pre = substitute_eqs(pre)
        post = substitute_eqs(post)

        # Create namespaces for pre and post codes
        pre_namespace = namespace(pre, level=level + 1)
        post_namespace = namespace(post, level=level + 1)

        def splitcode(incode):
            num_perneuron = num_persynapse = 0
            reordering_warning = False
            incode_lines = [line.strip() for line in incode.split('\n') if line.strip()]
            per_neuron_lines = []
            per_synapse_lines = []
            for line in incode_lines:
                if not line.strip(): continue
                m = re.search(r'\bw\b\s*[^><=]?=', line) # lines of the form w = ..., w *= ..., etc.
                if m:
                    num_persynapse += 1
                    per_synapse_lines.append(line)
                else:
                    num_perneuron += 1
                    if num_persynapse!=0 and not reordering_warning:
                        log_warn('brian.experimental.cstdp', 'STDP operations are being re-ordered, results may be wrong.')
                        reordering_warning = True
                    per_neuron_lines.append(line)
            return per_neuron_lines, per_synapse_lines

        per_neuron_pre, per_synapse_pre = splitcode(pre)
        per_neuron_post, per_synapse_post = splitcode(post)

        all_vars = vars_pre + vars_post + ['w']        

        per_neuron_pre = [c_single_statement(freeze(line, all_vars, pre_namespace)) for line in per_neuron_pre]
        per_neuron_post = [c_single_statement(freeze(line, all_vars, post_namespace)) for line in per_neuron_post]
        per_synapse_pre = [c_single_statement(freeze(line, all_vars, pre_namespace)) for line in per_synapse_pre]
        per_synapse_post = [c_single_statement(freeze(line, all_vars, post_namespace)) for line in per_synapse_post]

        per_neuron_pre = '\n'.join(per_neuron_pre)
        per_neuron_post = '\n'.join(per_neuron_post)
        per_synapse_pre = '\n'.join(per_synapse_pre)
        per_synapse_post = '\n'.join(per_synapse_post)

        # Neuron groups
        G_pre = NeuronGroup(len(C.source), model=sep_pre, clock=self.clock)
        G_post = NeuronGroup(len(C.target), model=sep_post, clock=self.clock)
        G_pre._S[:] = 0
        G_post._S[:] = 0
        self.pre_group = G_pre
        self.post_group = G_post
        var_group = {}
        for i, v in enumerate(vars_pre):
            var_group[v] = G_pre
        for i, v in enumerate(vars_post):
            var_group[v] = G_post
        self.var_group = var_group

        self.contained_objects += [G_pre, G_post]

        vars_pre_ind = {}
        for i, var in enumerate(vars_pre):
            vars_pre_ind[var] = i
        vars_post_ind = {}
        for i, var in enumerate(vars_post):
            vars_post_ind[var] = i

        prevars_dict = dict((k, G_pre.state(k)) for k in vars_pre)
        postvars_dict = dict((k, G_post.state(k)) for k in vars_post)

        clipcode = ''
        if isfinite(wmin):
            clipcode += 'if(w<%wmin%) w = %wmin%;\n'.replace('%wmin%', repr(float(wmin)))
        if isfinite(wmax):
            clipcode += 'if(w>%wmax%) w = %wmax%;\n'.replace('%wmax%', repr(float(wmax)))

        if not isinstance(C, DelayConnection):
            precode = iterate_over_spikes('_j', '_spikes',
                        (load_required_variables('_j', prevars_dict),
                         transform_code(per_neuron_pre),
                         iterate_over_row('_k', 'w', C.W, '_j',
                            (load_required_variables('_k', postvars_dict),
                             transform_code(per_synapse_pre),
                             ConnectionCode(clipcode)))))
            postcode = iterate_over_spikes('_j', '_spikes',
                        (load_required_variables('_j', postvars_dict),
                         transform_code(per_neuron_post),
                         iterate_over_col('_i', 'w', C.W, '_j',
                            (load_required_variables('_i', prevars_dict),
                             transform_code(per_synapse_post),
                             ConnectionCode(clipcode)))))
            log_debug('brian.experimental.c_stdp', 'CSTDP Pre code:\n' + str(precode))
            log_debug('brian.experimental.c_stdp', 'CSTDP Post code:\n' + str(postcode))
            connection_delay = C.delay * C.source.clock.dt
            if (delay_pre is None) and (delay_post is None): # same delays as the Connnection C
                delay_pre = connection_delay
                delay_post = 0 * ms
            elif delay_pre is None:
                delay_pre = connection_delay - delay_post
                if delay_pre < 0 * ms: raise AttributeError, "Postsynaptic delay is too large"
            elif delay_post is None:
                delay_post = connection_delay - delay_pre
                if delay_post < 0 * ms: raise AttributeError, "Postsynaptic delay is too large"
            # create forward and backward Connection objects or SpikeMonitor objects
            pre_updater = SpikeMonitor(C.source, function=precode, delay=delay_pre)
            post_updater = SpikeMonitor(C.target, function=postcode, delay=delay_post)
            updaters = [pre_updater, post_updater]
            self.contained_objects += [pre_updater, post_updater]
        else:
            if delay_pre is not None or delay_post is not None:
                raise ValueError("Must use delay_pre=delay_post=None for the moment.")
            max_delay = C._max_delay * C.target.clock.dt
            # Ensure that the source and target neuron spikes are kept for at least the
            # DelayConnection's maximum delay
            C.source.set_max_delay(max_delay)
            C.target.set_max_delay(max_delay)

            self.G_pre_monitors = {}
            self.G_post_monitors = {}
            self.G_pre_monitors.update(((var, RecentStateMonitor(G_pre, vars_pre_ind[var], duration=(C._max_delay + 1) * C.target.clock.dt, clock=G_pre.clock)) for var in vars_pre))
            self.G_post_monitors.update(((var, RecentStateMonitor(G_post, vars_post_ind[var], duration=(C._max_delay + 1) * C.target.clock.dt, clock=G_post.clock)) for var in vars_post))
            self.contained_objects += self.G_pre_monitors.values()
            self.contained_objects += self.G_post_monitors.values()

            prevars_dict_delayed = dict((k, self.G_pre_monitors[k]) for k in prevars_dict.keys())
            postvars_dict_delayed = dict((k, self.G_post_monitors[k]) for k in postvars_dict.keys())

            precode_immediate = iterate_over_spikes('_j', '_spikes',
                                    (load_required_variables('_j', prevars_dict),
                                     transform_code(per_neuron_pre)))
            precode_delayed = iterate_over_spikes('_j', '_spikes',
                                     iterate_over_row('_k', 'w', C.W, '_j', extravars={'_delay':C.delayvec},
                                        code=(
                                         ConnectionCode('double _t_past = _max_delay-_delay;', vars={'_max_delay':float(max_delay)}),
                                         load_required_variables_pastvalue('_k', '_t_past', postvars_dict_delayed),
                                         transform_code(per_synapse_pre),
                                         ConnectionCode(clipcode))))
            postcode = iterate_over_spikes('_j', '_spikes',
                            (load_required_variables('_j', postvars_dict),
                             transform_code(per_neuron_post),
                             iterate_over_col('_i', 'w', C.W, '_j', extravars={'_delay':C.delayvec},
                                code=(
                                 load_required_variables_pastvalue('_i', '_delay', prevars_dict_delayed),
                                 transform_code(per_synapse_post),
                                 ConnectionCode(clipcode)))))
            log_debug('brian.experimental.c_stdp', 'CSTDP Pre code (immediate):\n' + str(precode_immediate))
            log_debug('brian.experimental.c_stdp', 'CSTDP Pre code (delayed):\n' + str(precode_delayed))
            log_debug('brian.experimental.c_stdp', 'CSTDP Post code:\n' + str(postcode))
            pre_updater_immediate = SpikeMonitor(C.source, function=precode_immediate)
            pre_updater_delayed = SpikeMonitor(C.source, function=precode_delayed, delay=max_delay)
            post_updater = SpikeMonitor(C.target, function=postcode)
            updaters = [pre_updater_immediate, pre_updater_delayed, post_updater]
            self.contained_objects += updaters

    def __call__(self):
        pass

    def __getattr__(self, name):
        if name == 'var_group':
            # this seems mad - the reason is that getattr is only called if the thing hasn't
            # been found using the standard methods of finding attributes, which for var_index
            # should have worked, this is important because the next line looks for var_index
            # and if we haven't got a var_index we don't want to get stuck in an infinite
            # loop
            raise AttributeError
        if not hasattr(self, 'var_group'):
            # only provide lookup of variable names if we have some variable names, i.e.
            # if the var_index attribute exists
            raise AttributeError
        G = self.var_group[name]
        return G.state_(name)

    def __setattr__(self, name, val):
        if not hasattr(self, 'var_group') or name not in self.var_group:
            object.__setattr__(self, name, val)
        else:
            G = self.var_group[name]
            G.state_(name)[:] = val

if __name__ == '__main__':
    from time import time
    log_level_debug()

    structure = 'dense'
    delay = False

    if not delay:
        delay = 0 * ms

    max_delay = 5 * ms
    N = 1000
    taum = 10 * ms
    tau_pre = 20 * ms
    tau_post = tau_pre
    Ee = 0 * mV
    vt = -54 * mV
    vr = -60 * mV
    El = -74 * mV
    taue = 5 * ms
    F = 15 * Hz
    gmax = .01
    dA_pre = .01
    dA_post = -dA_pre * tau_pre / tau_post * 1.05

    eqs_neurons = '''
    dv/dt=(ge*(Ee-vr)+El-v)/taum : volt   # the synaptic current is linearized
    dge/dt=-ge/taue : 1
    '''

    input = PoissonGroup(N, rates=F)
    neurons = NeuronGroup(1, model=eqs_neurons, threshold=vt, reset=vr)
    synapses = Connection(input, neurons, 'ge', weight=rand(len(input), len(neurons)) * gmax,
                        structure=structure, delay=delay, max_delay=max_delay)
    neurons.v = vr

    #stdp=ExponentialSTDP(synapses,tau_pre,tau_post,dA_pre,dA_post,wmax=gmax)
    ## Explicit STDP rule
    eqs_stdp = '''
    dA_pre/dt=-A_pre/tau_pre : 1
    dA_post/dt=-A_post/tau_post : 1
    '''
    dA_post *= gmax
    dA_pre *= gmax
    stdp = CSTDP(synapses, eqs=eqs_stdp, pre='A_pre+=dA_pre;w+=A_post',
                 post='A_post+=dA_post;w+=A_pre', wmax=gmax)

    rate = PopulationRateMonitor(neurons)

    start_time = time()
    run(100 * second, report='text')
    print "Simulation time:", time() - start_time

    subplot(311)
    plot(rate.times / second, rate.smooth_rate(100 * ms))
    subplot(312)
    plot(synapses.W.todense() / gmax, '.')
    subplot(313)
    hist(synapses.W.todense() / gmax, 20)
    show()



from brian import *
set_global_preferences(usenewbrianhears=True,
                       useweave=False,
                       )
from brian.hears import *
#from brian.hears.filtering import GammatoneFilterbank as oldGFB

x = Sound(randn(1000, 1), samplerate=44100*Hz)

gfb = GammatoneFilterbank(x, [1*kHz, 1.01*kHz])
print gfb.nchannels
#old_gfb = oldGFB(44100*Hz, [1*kHz, 2*kHz])
#y2 = old_gfb.apply(asarray(x))

gfb.source = x*10000

ff = FunctionFilterbank(gfb, lambda input:clip(input, 0, Inf))

sfb = SumFilterbank((gfb, ff), (1, -1))

sfb = sfb*sfb

sfb.buffer_init()
y = sfb.buffer_fetch(0, 1500)

print y.shape

#for i in xrange(3):
#    y = gfb.buffer_fetch(i*500, i*500+1000)
#    
#    print (y.shape,
#           (gfb.cached_buffer_start,
#            gfb.cached_buffer_end,
#            gfb.cached_buffer_output.shape))

for i in xrange(sfb.nchannels):
    plot(y[:, i])
#    plot(y2[:, i])
show()



'''
Phase locking in leaky integrate-and-fire model
-----------------------------------------------
Fig. 2A from:
Brette R (2004). Dynamics of one-dimensional spiking neuron models.
J Math Biol 48(1): 38-56.

This shows the phase-locking structure of a LIF driven by a sinusoidal
current. When the current crosses the threshold (a<3), the model
almost always phase locks (in a measure-theoretical sense).
'''
from brian import *

#defaultclock.dt=0.01*ms # for a more precise picture
N=2000
tau=100*ms
freq=1/tau

eqs='''
dv/dt=(-v+a+2*sin(2*pi*t/tau))/tau : 1
a : 1
'''

neurons=NeuronGroup(N,eqs,threshold=1,reset=0)
neurons.a=linspace(2,4,N)

run(5*second,report='text') # discard the first spikes (wait for convergence)
S=SpikeMonitor(neurons)
run(5*second,report='text')

i,t=zip(*S.spikes)
plot((t % tau)/tau,i,'.')
xlabel('Spike phase')
ylabel('Parameter a')
yticks([0,N/2,N],[2,3,4])
show()



# ----------------------------------------------------------------------------------
# Copyright ENS, INRIA, CNRS
# Contributors: Romain Brette (brette@di.ens.fr) and Dan Goodman (goodman@di.ens.fr)
# 
# Brian is a computer program whose purpose is to simulate models
# of biological neural networks.
# 
# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use, 
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info". 
# 
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability. 
# 
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or 
# data to be ensured and,  more generally, to use and operate it in the 
# same conditions as regards security. 
# 
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.
# ----------------------------------------------------------------------------------
# 
'''
Circular arrays

Ideas for speed improvements: use put, putmask and take with mode='wrap' and out=...
'''
from numpy import *
from scipy import weave
import bisect
import os
import warnings
from ..globalprefs import get_global_preference

__all__ = ['CircularVector', 'SpikeContainer']


class CircularVector(object):
    '''
    A vector with circular structure.
    Variables:
    * X = the data (array of size n)
    * cursor = current position in the array (where the 0 index is)
    '''
    def __init__(self, n, dtype=float, useweave=False, compiler=None): # pylint: disable-msg=W0621
        '''
        n is the size of the vector.
        '''
        self.X = zeros(n, dtype=dtype)
        self.dtype = dtype
        self.cursor = 0
        self.n = n
        self._useweave = useweave
        if useweave:
            self._optimisedreturnarray = zeros(n, dtype=dtype)
            self._cpp_compiler = compiler
            self._extra_compile_args = ['-O3']
            if self._cpp_compiler == 'gcc':
                self._extra_compile_args += get_global_preference('gcc_options') # ['-march=native', '-ffast-math']
        else:
            self._cpp_compiler = ''

    def reinit(self):
        self.X[:] = zeros(self.n, self.dtype)
        self.cursor = 0

    def advance(self, k):
        self.cursor = (self.cursor + k) % self.n

    def __len__(self):
        return self.n

    def __getitem__(self, i):
        '''
        V[i]
        '''
        return self.X[(self.cursor + i) % self.n]

    def __setitem__(self, i, x):
        '''
        V[i]=x
        '''
        self.X[(self.cursor + i) % self.n] = x

    def __getslice__(self, i, j):
        n = self.n
        i0 = (self.cursor + i) % n # pylint: disable-msg=W0621
        j0 = (self.cursor + j) % n # pylint: disable-msg=W0621
        if j0 >= i0:
            return self.X[i0:j0]
        else:
            #return self.X[range(i0,n)+range(0,j0)]
            return concatenate((self.X[i0:], self.X[0:j0])) # this version is MUCH faster

    def get_conditional(self, i, j, min, max, offset=0):
        """
        Returns only those vectors with values between min and max
        
        This rather specialised usage of a circular vector is for the
        benefit of the SpikeContainer class get_spikes method, which is
        in turn used by the NeuronGroup.get_spikes method.
        
        It returns v-offset for those elements v in self[i:j] such that min<v<max. 
        """
        if self._useweave:
            n = self.n
            i0 = int((self.cursor + i) % n) # pylint: disable-msg=W0612,W0621
            j0 = int((self.cursor + j) % n) # pylint: disable-msg=W0612
            X = self.X # pylint: disable-msg=W0612
            ret = self._optimisedreturnarray
            code = """
                    int numgot = 0;
                    for(int k=i0;k!=j0;k=(k+1)%n)
                    {
                        int Xk = X(k);
                        if(Xk>=min && Xk<max)
                            ret(numgot++)=Xk-offset;
                    }
                    return_val = numgot;
                    """
            numgot = weave.inline(code, ['n', 'i0', 'j0', 'X', 'ret', 'offset', 'min', 'max'],
                                     compiler=self._cpp_compiler,
                                     type_converters=weave.converters.blitz,
                                     extra_compile_args=self._extra_compile_args)
            return ret[0:numgot]
        else:
            spikes = self[i:j]
            spikes = spikes[bisect.bisect_left(spikes, min):bisect.bisect_left(spikes, max)]
            if offset: spikes = spikes - offset
            return spikes

    def __setslice__(self, i, j, W):
        # NB: S[4:1] does not give a circular slice but only []
        # Should we change that behaviour?
        # TODO: speed improvements
        if j > i:
            n = self.n
            i0 = (self.cursor + i) % n # pylint: disable-msg=W0621
            j0 = (self.cursor + j) % n
            if j0 > i0:
                self.X[i0:j0] = W
            elif isinstance(W, ndarray):
                self.X[i0:] = W[0:n - i0]
                self.X[0:j0] = W[n - i0:n - i0 + j0]

    def __repr__(self):
        return repr(hstack((self[0:self.n - 1], self[self.n - 1:self.n])))

    def __print__(self):
        return (hstack((self[0:self.n - 1], self[self.n - 1:self.n]))).__print__()


class SpikeContainer(object):
    '''
    An object that stores previous spikes.
    S[0] is an array of the last spikes (neuron indexes).
    S[1] is an array with the spikes at time t-dt, etc.
    S[0:50] contains all spikes in last 50 bins.
    '''
    def __init__(self, m, useweave=False, compiler=None):
        '''
        n = maximum number of spikes stored (not used anymore)
        m = maximum number of bins stored
        '''
        if m < 2: m = 2
        self.S = CircularVector(2, dtype=int, useweave=useweave, compiler=compiler)
        self.ind = CircularVector(m + 1, dtype=int, useweave=useweave, compiler=compiler) # indexes of bins
        self.remaining_space = 1
        self._useweave = useweave

    def reinit(self):
        self.S.reinit()
        self.ind.reinit()

    def push(self, spikes):
        '''
        Stores spikes in the array at time dt.
        '''
        ns = len(spikes)
        self.remaining_space += (self.ind[2] - self.ind[1]) % self.S.n
        while ns >= self.remaining_space:
            # double size of array
            S = self.S
            newS = CircularVector(2 * S.n, dtype=int, useweave=S._useweave, compiler=S._cpp_compiler)
            newS.X[:S.n - S.cursor] = S.X[S.cursor:]
            newS.X[S.n - S.cursor:S.n] = S.X[:S.cursor]
            newS.cursor = S.n
            self.S = newS
            self.ind.X = (self.ind.X - S.cursor) % S.n
            self.ind.X[self.ind.X == 0] = S.n
            self.remaining_space += S.n
        self.S[0:ns] = spikes
        self.S.advance(ns)
        self.ind.advance(1)
        self.ind[0] = self.S.cursor
        self.remaining_space -= ns

    def lastspikes(self):
        '''
        Returns S[0].
        '''
        return self.S[self.ind[-1] - self.S.cursor:self.S.n]

    def __getitem__(self, i):
        '''
        S[i]: returns the spikes at time t-i*dt.
        '''
        # NB: this could be optimized
        return self.S[self.ind[-i - 1] - self.S.cursor:self.ind[-i] - self.S.cursor + self.S.n]

    # optimised version of the above, but the speed improvement is not very much, might be
    # better to just wait and write a fully C/C++ version of the whole library
    def get_spikes(self, delay, origin, N):
        """
        Returns those spikes in self[delay] between origin and origin+N
        """
        return self.S.get_conditional(self.ind[-delay - 1] - self.S.cursor, \
                                     self.ind[-delay] - self.S.cursor + self.S.n, \
                                     origin, origin + N, origin)

    def __getslice__(self, i, j):
        return self.S[self.ind[-j] - self.S.cursor:self.ind[-i] - self.S.cursor + self.S.n]

    def __repr__(self):
        return "Spike container."

    def __print__(self):
        return self.__repr__()

try:
    import ccircular.ccircular as _ccircular
    class SpikeContainer(_ccircular.SpikeContainer):
        def __init__(self, m, useweave=False, compiler=None):
            _ccircular.SpikeContainer.__init__(self, m)
    #warnings.warn('Using C++ SpikeContainer')
except ImportError:
    pass

# I am not sure that class below is useful!
class ModInt(object):
    '''
    A number in Z/nZ, i.e., modulo n.
    Variables:
    * x = number modulo n
    * n
    Implemented: additions and subtraction.
    N.B.: not a very useful class.
    '''
    def __init__(self, x, n):
        self.x = x
        self.n = n

    def __add__(self, m):
        if isinstance(m, ModInt):
            assert self.n == m.n
            return ModInt((self.x + m.x) % self.n, self.n)
        else:
            return ModInt((self.x + m) % self.n, self.n)

    def __radd__(self, m):
        if isinstance(m, ModInt):
            assert self.n == m.n
            return ModInt((self.x + m.x) % self.n, self.n)
        else:
            return ModInt((self.x + m) % self.n, self.n)

    def __sub__(self, m):
        if isinstance(m, ModInt):
            assert self.n == m.n
            return ModInt((self.x - m.x) % self.n, self.n)
        else:
            return ModInt((self.x - m) % self.n, self.n)

    def __repr__(self):
        return str(self.x) + ' [' + str(self.n) + ']'

    def __print__(self):
        return self.__repr__()



'''
Topographic map - an example of complicated connections.
Two layers of neurons.
The first layer is connected randomly to the second one in a
topographical way.
The second layer has random lateral connections.
'''
from brian import *

N = 100
tau = 10 * ms
tau_e = 2 * ms # AMPA synapse
eqs = '''
dv/dt=(I-v)/tau : volt
dI/dt=-I/tau_e : volt
'''

rates = zeros(N) * Hz
rates[N / 2 - 10:N / 2 + 10] = ones(20) * 30 * Hz
layer1 = PoissonGroup(N, rates=rates)
layer2 = NeuronGroup(N, model=eqs, threshold=10 * mV, reset=0 * mV)

topomap = lambda i, j:exp(-abs(i - j) * .1) * 3 * mV
feedforward = Connection(layer1, layer2, sparseness=.5, weight=topomap)
#feedforward[2,3]=1*mV

lateralmap = lambda i, j:exp(-abs(i - j) * .05) * 0.5 * mV
recurrent = Connection(layer2, layer2, sparseness=.5, weight=lateralmap)

spikes = SpikeMonitor(layer2)

run(1 * second)
subplot(211)
raster_plot(spikes)
subplot(223)
imshow(feedforward.W.todense(), interpolation='nearest', origin='lower')
title('Feedforward connection strengths')
subplot(224)
imshow(recurrent.W.todense(), interpolation='nearest', origin='lower')
title('Recurrent connection strengths')
show()



# SOLUTION 2: DERIVED CLASS

from brian import *
from brian.directcontrol import MultipleSpikeGeneratorThreshold


class MultipleSpikeGeneratorGroupWithDelays(MultipleSpikeGeneratorGroup):
    def __init__(self, spiketimes, clock=None, max_delay=0 * ms):
        clock = guess_clock(clock)
        thresh = MultipleSpikeGeneratorThreshold(spiketimes)
        NeuronGroup.__init__(self, len(spiketimes), model=LazyStateUpdater(), threshold=thresh, clock=clock, max_delay=max_delay)

tau = 10 * ms
sigma = 5 * mV
eqs = 'dv/dt = -v/tau : volt'
pop = NeuronGroup(1, model=eqs, threshold=10 * mV, reset=0 * mV, refractory=5 * ms)
input = MultipleSpikeGeneratorGroupWithDelays([[0.1 * second, 0.2 * second, 0.3 * second, 0.4 * second, 0.5 * second]], max_delay=3 * ms)
pop.v = -60 * mV
C = Connection(input, pop, 'v', delay=2 * ms)
C.connect_one_to_one(input, pop, weight=15 * mV)
Minp = SpikeMonitor(input)
M = SpikeMonitor(pop, True)
run(1 * second)
print Minp.spikes
print M.spikes


# SOLUTION 1: INTERMEDIATE GROUP
#
#from brian import *
#tau=10*ms
#sigma=5*mV
#eqs='dv/dt = -v/tau : volt'
#pop=NeuronGroup(1,model=eqs,threshold=10*mV,reset=0*mV,refractory=5*ms)
#inputgen=MultipleSpikeGeneratorGroup([[0.1*second,0.2*second,0.3*second,0.4*second,0.5*second]])
#input=NeuronGroup(len(inputgen),model='v:1',threshold=0.5,reset=0.,max_delay=3*ms)
#C_inputgen_input=IdentityConnection(inputgen,input)
#pop.v=-60*mV
#C=Connection(input,pop,'v',delay=2*ms-defaultclock.dt)
#C.connect_one_to_one(input, pop, weight=15*mV)
#Minpgen = SpikeMonitor(inputgen)
#Minp = SpikeMonitor(input)
#M=SpikeMonitor(pop,True)
#run(1*second)
#print Minpgen.spikes
#print Minp.spikes
#print M.spikes



'''
NOTES:

General scheme for propagation code is something like:

Iterate over spikes (spike index i):
    Set source neuron index j=spikes[i]
    
    Load required source neuron variables for neuron index j
    
    Iterate over row j of W:
        Set target neuron index k
        Load weight variable
        
        Load required target neuron variables for neuron index k
        
        Execute propagation code

Functionally:

iterate_over_spikes('j', 'spikes',
    (load_required_variables('j', {'modulation':modulation_var}),
     iterate_over_row('k', 'w', W, 'j',
         (load_required_variables('k', {'V':V}),
          transform_code('V += w*modulation')
          )
         )
     )
    )

With some options like:

* load_required_variables('j', {})
* iterate_over_row('k', 'w', W, 'j', delayvec=delayvec, delayvar='delay', ...)
* load_required_variables_delayedreaction('k', {'V':{'dr':dr, 'cdi':cdi, 'idt':idt, 'md':md}}) 

We could also have:

* iterate_over_col('k', 'w', W, 'j', ...)

And STDP could be implemented as follows, for pre spikes:

iterate_over_spikes('j', 'spikes',
    (load_required_variables('j', {'A_pre':A_pre}),
     """
     A_pre += dA_pre
     """,
     iterate_over_row('k', 'w', W, 'j',
         (load_required_variables('k', {'A_post':A_post}),
          transform_code('w += A_post')
          )
         )
     )
    )

And for post spikes:

iterate_over_spikes('j', 'spikes',
    (load_required_variables('j', {'A_post':A_post}),
     transform_code('A_post += dA_post'),
     iterate_over_col('k', 'w', W, 'j',
         (load_required_variables('k', {'A_pre':A_pre}),
          transform_code('w += A_pre')
          )
         )
     )
    )

To do STDP with delays, we need also:

* load_required_variables_pastvalue('k', {'A_post':A_post_monitor})

Maybe for future-proofing STDP against having multiple per-synapse variables in
the future, and generally having per-synapse dynamics and linked matrices which
we want to jointly iterate over, could improve iterate_over_row/col to iterate
over several synaptic variables with the same underlying matrix structure as the
main weight matrix. Then, instead of having delayvec=delayvec, delayvar='delay'
as a special case, we'd have a list of additional linked matrices.

TODO:

* Have ConnectionCode object feature a C and Python code and namespace to
  be executed. The Python namespace can be used for grabbing some parameters
  such as the _cdi variables that currently have to be set in by hand.
'''

if __name__ == '__main__':
    from brian import *
    from brian.utils.documentation import *
    from brian.experimental.codegen.c_support_code import *
else:
    from ..connections import Connection, DelayConnection, MultiConnection, \
                DenseConnectionMatrix, DenseConstructionMatrix, \
                SparseConnectionMatrix, SparseConstructionMatrix, \
                DynamicConnectionMatrix, DynamicConstructionMatrix
    from ..log import log_debug, log_warn, log_info
    from ..utils.documentation import flattened_docstring
    from ..globalprefs import get_global_preference
    from codegen.c_support_code import *
import numpy
from scipy import weave
import new

__all__ = ['make_new_connection',
           'expand_code', 'transform_code',
           'iterate_over_spikes',
           'load_required_variables',
           'load_required_variables_delayedreaction',
           'load_required_variables_pastvalue',
           'iterate_over_row',
           'iterate_over_col',
           'ConnectionCode',
           ]


class ConnectionCode(object):
    def __init__(self, codestr, vars=None, pycodestr=None, pyvars=None):
        if pyvars is None:
            pyvars = {}
        if pycodestr is None:
            pycodestr = ''
        if vars is None:
            vars = {}
        self.vars = vars
        self.codestr = codestr
        if len([line for line in pycodestr.split('\n') if line]) > 1:
            pycodestr = flattened_docstring(pycodestr)
        else:
            pycodestr = pycodestr.strip()
        self.pycodestr = pycodestr
        self.pyvars = pyvars
        self.prepared = False
        self._weave_compiler = get_global_preference('weavecompiler')
        self._extra_compile_args = ['-O3']
        if self._weave_compiler == 'gcc':
            self._extra_compile_args += get_global_preference('gcc_options') # ['-march=native', '-ffast-math']

    def prepare(self):
        self.pyvars['vars'] = self.vars
        self.vars['_spikes'] = None
        self.vars['_spikes_len'] = None
        self.vars_list = self.vars.keys()
        if len(self.pycodestr):
            self.compiled_pycode = compile(self.pycodestr, 'ConnectionCode', 'exec')
        else:
            self.compiled_pycode = None
        self.prepared = True

    def __call__(self, _spikes):
        if not self.prepared:
            self.prepare()
        if len(_spikes):
            if not isinstance(_spikes, numpy.ndarray):
                _spikes = array(_spikes, dtype=int)
            vars = self.vars
#            print '****'
#            print self.codestr
#            for k, v in vars.iteritems():
#                if isinstance(v, numpy.ndarray):
#                    print k, ': shape =', v.shape
#                else:
#                    print k, ':', v
#            import sys
#            sys.stdout.flush()
            vars['_spikes'] = _spikes
            vars['_spikes_len'] = len(_spikes)
            if self.compiled_pycode is not None:
                exec self.compiled_pycode in self.pyvars
            weave.inline(self.codestr, self.vars_list,
                         local_dict=self.vars,
                         support_code=c_support_code,
                         compiler=self._weave_compiler,
                         extra_compile_args=self._extra_compile_args)

    def __str__(self):
        s = 'C code:\n'
        spaces = 0
        for line in self.codestr.split('\n'):
            if line.strip():
                if '}' in line: spaces -= 4
                s += ' ' * spaces + line.strip() + '\n'
                if '{' in line: spaces += 4
        s += 'Python code:\n'
        s += self.pycodestr
        return s
    __repr__ = __str__

def expand_code(code):
    if isinstance(code, ConnectionCode):
        return code
    elif isinstance(code, (tuple, list)):
        codestr = '\n'.join([expand_code(c).codestr for c in code])
        vars = {}
        pyvars = {}
        for c in code:
            vars.update(c.vars)
            pyvars.update(c.pyvars)
        pycodestr = '\n'.join([expand_code(c).pycodestr for c in code])
        return ConnectionCode(codestr, vars, pycodestr, pyvars)
    else:
        raise TypeError('ConnectionCode should be string or tuple')

def transform_code(codestr, vars=None):
    # TODO: replace with something more sophisticated than this
    return ConnectionCode('\n'.join(line + ';' for line in codestr.split('\n') if line.strip()), vars)

def iterate_over_spikes(neuron_index, spikes, code):
    outcode = '''
    for(int _spike_index=0; _spike_index<%SPIKES_LEN%; _spike_index++)
    {
        const int %NEURON_INDEX% = %SPIKES%[_spike_index];
        %CODE%
    }
    '''
    code = expand_code(code)
    outcode = outcode.replace('%SPIKES%', spikes)
    outcode = outcode.replace('%SPIKES_LEN%', spikes + '_len')
    outcode = outcode.replace('%NEURON_INDEX%', neuron_index)
    outcode = outcode.replace('%CODE%', code.codestr)
    return ConnectionCode(outcode, code.vars, code.pycodestr, code.pyvars)

def load_required_variables(neuron_index, neuron_vars):
    vars = {}
    codestr = ''
    for k, v in neuron_vars.iteritems():
        vars[k + '__array'] = v
        codestr += 'double &' + k + ' = ' + k + '__array[' + neuron_index + '];\n'
    return ConnectionCode(codestr, vars)

def load_required_variables_delayedreaction(neuron_index, delay, delay_index, neuron_var, C):
    vars = {}
    codestr = 'double &%Z% = _dr[((%CDI%+(int)(_idt*%D%))%_md)*%N%+%I%];'
    codestr = codestr.replace('%CDI%', delay_index)
    codestr = codestr.replace('%Z%', neuron_var)
    codestr = codestr.replace('%D%', delay)
    codestr = codestr.replace('%N%', '_num_target_neurons')
    codestr = codestr.replace('%I%', neuron_index)
    vars['_num_target_neurons'] = len(C.target)
    vars['_dr'] = C._delayedreaction
    vars[delay_index] = None # filled in by Python code (below)
    vars['_idt'] = C._invtargetdt
    vars['_md'] = C._max_delay
    pyvars = {'conn':C}
    pycodestr = "vars['_cdi'] = conn._cur_delay_ind"
    return ConnectionCode(codestr, vars, pycodestr, pyvars)

def load_required_variables_pastvalue(neuron_index, time, neuron_vars):
    vars = {}
    pyvars = {}
    pycodestr = ''
    codestr = ''
    for k, M in neuron_vars.iteritems():
        vars[k + '__values'] = M._values
        vars[k + '__arraylen'] = M._values.shape[1]
        vars[k + '__cti'] = None # current_time_index filled in by propagation function
        vars[k + '__idt'] = M._invtargetdt
        vars[k + '__nd'] = M.num_duration
        pyvars[k + '__RecentStateMonitor'] = M
        pycodestr += "vars['%k%__cti'] = %k%__RecentStateMonitor.current_time_index\n" .replace('%k%', k)
        newcodestr = 'double &%var% = %var%__values[((%var%__nd+%var%__cti-1-(int)(%var%__idt*%time%))%%var%__nd)*%var%__arraylen+%i%];\n'
        newcodestr = newcodestr.replace('%var%', k)
        newcodestr = newcodestr.replace('%time%', time)
        newcodestr = newcodestr.replace('%i%', neuron_index)
        codestr += newcodestr;
    return ConnectionCode(codestr, vars, pycodestr, pyvars)

def iterate_over_row(target_index, weight_variable, weight_matrix, source_index,
                     code, extravars={}):
    code = expand_code(code)
    vars = {}
    vars.update(code.vars)
    if isinstance(weight_matrix, DenseConnectionMatrix):
        outcode = '''
        double *_weight_arr_row = _weight_arr+%SOURCEINDEX%*_num_target_neurons;
        for(int %TARGETINDEX%=0; %TARGETINDEX%<_num_target_neurons; %TARGETINDEX%++)
        {
            double &%WEIGHT% = _weight_arr_row[%TARGETINDEX%];
            %EXTRAVARS%
            %CODE%
        }
        '''
        extravarscode = ''
        for k, v in extravars.iteritems():
            extracodetmp = 'double &%V% = %V%__array[%TARGETINDEX%+%SOURCEINDEX%*_num_target_neurons];'
            extracodetmp = extracodetmp.replace('%V%', k)
            extracodetmp = extracodetmp.replace('%TARGETINDEX%', target_index)
            extracodetmp = extracodetmp.replace('%SOURCEINDEX%', source_index)
            vars[k + '__array'] = numpy.asarray(v)
            extravarscode += extracodetmp
        outcode = outcode.replace('%EXTRAVARS%', extravarscode)
        vars['_weight_arr'] = numpy.asarray(weight_matrix)
        vars['_num_target_neurons'] = weight_matrix.shape[1]
    elif isinstance(weight_matrix, SparseConnectionMatrix):
        outcode = '''
        for(int _p=_rowind[%SOURCEINDEX%]; _p<_rowind[%SOURCEINDEX%+1]; _p++)
        {
            int %TARGETINDEX% = _allj[_p];
            double &%WEIGHT% = _alldata[_p];
            %EXTRAVARS%
            %CODE%
        }
        '''
        extravarscode = ''
        for k, v in extravars.iteritems():
            extracodetmp = 'double &%V% = %V%__alldata[_p];'
            extracodetmp = extracodetmp.replace('%V%', k)
            vars[k + '__alldata'] = v.alldata
            extravarscode += extracodetmp
        outcode = outcode.replace('%EXTRAVARS%', extravarscode)
        vars['_rowind'] = weight_matrix.rowind
        vars['_allj'] = weight_matrix.allj
        vars['_alldata'] = weight_matrix.alldata
    elif isinstance(weight_matrix, DynamicConnectionMatrix):
        # TODO: support dynamic matrix structure
        # the best way to support dynamic matrix type would be to
        # reorganise dynamic matrix data structure. Ideally, it should consist
        # of numpy arrays only. Maybe some sort of linked list structure?
        # Otherwise, we can use code that accesses the Python lists, but it's
        # probably less efficient (maybe this is anyway not a big issue with
        # the dynamic matrix type?)
        raise TypeError('Dynamic matrix not supported.')
    else:
        raise TypeError('Must be dense/sparse/dynamic matrix.')
    outcode = outcode.replace('%TARGETINDEX%', target_index)
    outcode = outcode.replace('%SOURCEINDEX%', source_index)
    outcode = outcode.replace('%WEIGHT%', weight_variable)
    outcode = outcode.replace('%CODE%', code.codestr)
    return ConnectionCode(outcode, vars, code.pycodestr, code.pyvars)

def iterate_over_col(source_index, weight_variable, weight_matrix, target_index,
                     code, extravars={}):
    code = expand_code(code)
    vars = {}
    vars.update(code.vars)
    if isinstance(weight_matrix, DenseConnectionMatrix):
        outcode = '''
        double *_weight_arr_row = _weight_arr+%TARGETINDEX%;
        for(int %SOURCEINDEX%=0; %SOURCEINDEX%<_num_source_neurons; %SOURCEINDEX%++)
        {
            double &%WEIGHT% = _weight_arr_row[%SOURCEINDEX%*_num_target_neurons];
            %EXTRAVARS%
            %CODE%
        }
        '''
        extravarscode = ''
        for k, v in extravars.iteritems():
            extracodetmp = 'double &%V% = %V%__array[%SOURCEINDEX%+%TARGETINDEX%*_num_target_neurons];'
            extracodetmp = extracodetmp.replace('%V%', k)
            extracodetmp = extracodetmp.replace('%SOURCEINDEX%', source_index)
            extracodetmp = extracodetmp.replace('%TARGETINDEX%', target_index)
            vars[k + '__array'] = numpy.asarray(v)
            extravarscode += extracodetmp
        outcode = outcode.replace('%EXTRAVARS%', extravarscode)
        vars['_weight_arr'] = numpy.asarray(weight_matrix)
        vars['_num_source_neurons'] = weight_matrix.shape[0]
        vars['_num_target_neurons'] = weight_matrix.shape[1]
    elif isinstance(weight_matrix, SparseConnectionMatrix):
        outcode = '''
        for(int _q=_colind[%TARGETINDEX%]; _q<_colind[%TARGETINDEX%+1]; _q++)
        {
            int _p = _allcoldataindices[_q];
            int %SOURCEINDEX% = _colalli[_q];
            double &%WEIGHT% = _alldata[_p];
            %EXTRAVARS%
            %CODE%
        }
        '''
        extravarscode = ''
        for k, v in extravars.iteritems():
            extracodetmp = 'double &%V% = %V%__alldata[_p];'
            extracodetmp = extracodetmp.replace('%V%', k)
            vars[k + '__alldata'] = v.alldata
            extravarscode += extracodetmp
        outcode = outcode.replace('%EXTRAVARS%', extravarscode)
        vars['_colind'] = weight_matrix.colind
        vars['_colalli'] = weight_matrix.colalli
        vars['_allcoldataindices'] = weight_matrix.allcoldataindices
        vars['_alldata'] = weight_matrix.alldata
    elif isinstance(weight_matrix, DynamicConnectionMatrix):
        # TODO: support dynamic matrix structure
        # the best way to support dynamic matrix type would be to
        # reorganise dynamic matrix data structure. Ideally, it should consist
        # of numpy arrays only. Maybe some sort of linked list structure?
        # Otherwise, we can use code that accesses the Python lists, but it's
        # probably less efficient (maybe this is anyway not a big issue with
        # the dynamic matrix type?)
        raise TypeError('Dynamic matrix not supported.')
    else:
        raise TypeError('Must be dense/sparse/dynamic matrix.')
    outcode = outcode.replace('%SOURCEINDEX%', source_index)
    outcode = outcode.replace('%TARGETINDEX%', target_index)
    outcode = outcode.replace('%WEIGHT%', weight_variable)
    outcode = outcode.replace('%CODE%', code.codestr)
    return ConnectionCode(outcode, vars, code.pycodestr, code.pyvars)

# TODO:
# * TEST iterate_over_col
# * TEST load_required_variables_pastvalue

def generate_connection_code(C):
    modulation = C._nstate_mod is not None
    delay = isinstance(C, DelayConnection)
    if not modulation and not delay:
        code = iterate_over_spikes('_j', '_spikes',
                     iterate_over_row('_k', 'w', C.W, '_j',
                         (load_required_variables('_k', {'V':C.target.state(C.nstate)}),
                          transform_code('V += w'))))
    elif modulation and not delay:
        code = iterate_over_spikes('_j', '_spikes',
                     (load_required_variables('_j', {'modulation':C.source.state(C._nstate_mod)}),
                      iterate_over_row('_k', 'w', C.W, '_j',
                         (load_required_variables('_k', {'V':C.target.state(C.nstate)}),
                          transform_code('V += w*modulation')))))
    elif not modulation and delay:
        code = iterate_over_spikes('_j', '_spikes',
                     iterate_over_row('_k', 'w', C.W, '_j', extravars={'_delay':C.delayvec},
                         code=(load_required_variables_delayedreaction('_k', '_delay', '_cdi', 'V', C),
                               transform_code('V += w'))))
    elif modulation and delay:
        code = iterate_over_spikes('_j', '_spikes',
                     (load_required_variables('_j', {'modulation':C.source.state(C._nstate_mod)}),
                      iterate_over_row('_k', 'w', C.W, '_j', extravars={'_delay':C.delayvec},
                         code=(load_required_variables_delayedreaction('_k', '_delay', '_cdi', 'V', C),
                               transform_code('V += w*modulation')))))
    else:
        raise TypeError('Not supported.')
    return code

def new_propagate(self, _spikes):
    if not self.iscompressed:
        self.compress()
    if not hasattr(self, '_connection_code'):
        self._connection_code = generate_connection_code(self)
        log_warn('brian.experimental.new_c_propagate', 'Using new C based propagation function.')
        log_debug('brian.experimental.new_c_propagate', 'C based propagation code:\n' + str(self._connection_code))
    if len(_spikes):
        self._connection_code(_spikes)

def make_new_connection(C):
    if C.__class__ is MultiConnection:
        for c in C.connections:
            make_new_connection(c)
    if C.__class__ is Connection or C.__class__ is DelayConnection:
        if C.W.__class__ is SparseConnectionMatrix or \
           C.W.__class__ is DenseConnectionMatrix or \
           C.W.__class__ is SparseConstructionMatrix or \
           C.W.__class__ is DenseConstructionMatrix:
            C.propagate = new.instancemethod(new_propagate, C, C.__class__)

if __name__ == '__main__':

    log_level_debug()

    structure = 'sparse'
    delay = 0 * ms#0*ms or True
    modulation = False

    G = NeuronGroup(1, 'V:1\nmod:1', reset=0, threshold=1)
    G.V = 2
    G.mod = 5
    H = NeuronGroup(10, 'V:1')
    if modulation:
        modulation = 'mod'
    else:
        modulation = None
    C = Connection(G, H, 'V', structure=structure, delay=delay, modulation=modulation)
    C[0, :] = linspace(0, 1, 10)
    if delay:
        C.delay[0, :] = linspace(0, 1, 10) * ms

    M = StateMonitor(H, 'V', record=True)

    make_new_connection(C)

    run(2 * ms)

    M.plot()
    legend()
    show()



from clustertools_example_manager import *

if __name__ == '__main__':
    cluster_worker_script(work_class,
                          max_gpu=1, max_cpu=3,
                          named_pipe=True,
                          )



"""
The basic Code object
"""
from brian import *
from brian.globalprefs import get_global_preference

__all__ = ['Code',
           'PythonCode',
           'CCode',
           'GPUCode',
           ]

class Code(object):
    '''
    The basic Code object used for all Python/C/GPU code generation.
    
    The Code object has the following attributes:
    
    ``code_str``
        A representation of the code in string form
    ``namespace``
        A dictionary of name/value pairs in which the code will be
        executed
    ``code_compiled``
        An optional value (can be None) consisting of some representation of the
        compiled form of the code
    ``pre_code``, ``post_code``
        Two optional Code objects which can be in the same or different
        languages, and can share partially or wholly the namespace. They are
        called (respectively) before or after the current code object is
        executed.
        
    Each language (e.g. PythonCode) extends some or all of the methods::
    
    ``__init__(...)``
        Unsurprisingly used for initialising the object, should call
        Code.__init__ with all of its arguments.
    ``compile()``
        Compiles the code, if necessary. If not necessary, set the
        ``code_compiled`` value to any dummy value other than None.
    ``run()``
        Runs the compiled code in the namespace.
    
    It will usually not be necessary to override the call mechanism:
    
    ``__call__(**kwds)``
        Calls ``pre_code(**kwds)``, updates the namespace with ``kwds``,
        executes the code (calls ``self.run()``) and then calls
        ``post_code(**kwds)``.
    '''
    def __init__(self, code_str, namespace, pre_code=None, post_code=None):
        self.code_str = code_str
        self.namespace = namespace
        self.code_compiled = None
        self.pre_code = pre_code
        self.post_code = post_code
    def compile(self):
        pass
    def run(self):
        pass
    def __call__(self, **kwds):
        if self.pre_code is not None:
            self.pre_code(**kwds)
        self.namespace.update(kwds)
        if self.code_compiled is None:
            self.compile()
        self.run()
        if self.post_code is not None:
            self.post_code(**kwds)


class PythonCode(Code):
    def compile(self):
        self.code_compiled = compile(self.code_str, 'PythonCode()')
    def run(self):
        exec self.code_compiled in self.namespace


class CCode(Code):
    def __init__(self, code_str, namespace, pre_code=None, post_code=None):
        Code.__init__(self, code_str, namespace, pre_code=pre_code,
                      post_code=post_code)
        self._weave_compiler = get_global_preference('weavecompiler')
        self._extra_compile_args = ['-O3']
        if self._weave_compiler == 'gcc':
            self._extra_compile_args += get_global_preference('gcc_options')
        self.code_compiled = 0
    def run(self):
        weave.inline(self.code_str, self.namespace.keys(),
                     local_dict = self.namespace,
                     #support_code=c_support_code,
                     compiler=self._weave_compiler,
                     extra_compile_args=self._extra_compile_args)


class GPUCode(Code):
    pass

if __name__=='__main__':
    pass


speedtrack_desc = 'Connection speed (spiking every dt, full connectivity)'

from brian import *
from tracking import *
import gc

num_neurons_set = [10, 20, 50, 100]
structure_set = ['sparse', 'dense', 'dynamic']

def run_barrage():
    results = []
    for n in num_neurons_set:
        for structure in structure_set:
            t = TrackSpeed(n, structure)
            results.append((str(n) + ' (' + structure + ')', track(t), (n, structure)))
            gc.collect()
    return (speedtrack_desc, results)

def plot_results(name, results):
    for structure in structure_set:
        allt = [t for desc, t, args in results if args[1] == structure]
        alln = [args[0] for desc, t, args in results if args[1] == structure]
        plot(alln, allt, label=structure)
    legend()
    title(name)


class TrackSpeed(object):
    def __init__(self, num_neurons, structure='sparse'):
        s = arange(num_neurons)
        class thr(Threshold):
            def __call__(self, P):
                return s
        G = NeuronGroup(num_neurons, 'V:1', threshold=thr())
        H = NeuronGroup(num_neurons, 'V:1')
        C = Connection(G, H, structure=structure)
        C.connect_full(G, H, weight=1)
        self.net = Network(G, H, C)
        self.net.prepare()

    def run(self):
        self.net.run(1 * second)

if __name__ == '__main__':
    name, results = run_barrage()
    print name
    for desc, t, args in results:
        print desc, ':', t
    plot_results(name, results)
    show()



from brian import *

#tau_pre = 5*ms
tau_post = 5 * ms
gmax = 1.0
delta_A_pre = 0.2
delta_A_post = -0.3

G = NeuronGroup(2, 'V:1', reset=0, threshold=1)

C = Connection(G, G, 'V')

C[0, 1] = 0.5

stdp = STDP(C, '''
            dA_pre/dt  = -0.1*sqrt(clip(A_pre,0,inf))/tau_pre   : 1
            dA_post/dt = -A_post/tau_post : 1
            tau_pre : second
            ''', pre='''
            A_pre += delta_A_pre
            w += A_post
            ''', post='''
            A_post += delta_A_post
            w += A_pre
            ''', wmax=gmax)

stdp.tau_pre = [2 * ms, .1 * ms]

M_pre = StateMonitor(stdp.pre_group, 'A_pre', record=True)
M_post = StateMonitor(stdp.post_group, 'A_post', record=True)

wvals = []

@network_operation
def rec_wvals():
    wvals.append(C[0, 1])

run(1 * ms)
G.V[1] = 2
run(1 * ms)
G.V[0] = 2
run(1 * ms)

subplot(221)
M_pre.plot()
subplot(222)
M_post.plot()
subplot(223)
plot(wvals)

show()



from numpy import empty, ndarray, exp
from fastexp import fastexp as _fastexp

def fastexp(x, out=None):
    if not isinstance(x, ndarray) or len(x.shape) > 1:
        return exp(x)
    if out is None:
        y = empty(x.shape)
        _fastexp(x, y)
        return y
    _fastexp(x, out)
    return out



#!/usr/bin/env python
from brian import *
from brian.library.IF import *
from general_stdp import *
from heterog_delay import *

N = 3
T = 500 * ms
max_delay = 20 * ms
vm0 = -65 * mV
w0 = 0.2 * vm0 / ms
Vt = -30 * mV
input_freq = 20 * Hz
input_curr = 20 * mV

#STDP
gmax = 0.010
tau_pre = 20 * ms
tau_post = 20 * ms
dA_pre = gmax * .005
dA_post = -dA_pre * 1.05

eqs = Izhikevich(a=0.02 / ms, b=0.2 / ms)
reset = AdaptiveReset(Vr= -75 * mV, b=8.0 * nA)
model = Model(model=eqs, threshold=Vt, reset=reset, max_delay=max_delay)

G = N * model
G.vm = vm0
G.w = w0

def nextspike():
    interval = 1 / input_freq
    nexttime = interval
    while True:
        yield (0, nexttime)
        nexttime += interval


input = SpikeGeneratorGroup(1, nextspike())
Ic = Connection(input, G)
C1 = Connection(G, G, structure="dense", delay=1 * ms)
C2 = Connection(G, G, structure="dense", delay=2 * ms)
Ic[0, 0] = input_curr
Ic[0, 1] = input_curr

#for i in range(len(G)):
#    C1[i]=[0.0]*len(G)
#    C2[i]=[0.0]*len(G)

C1[0, 2] = 0.009
C2[1, 2] = 0.009


stdp1 = SongAbbottSTDP(C1, gmax=gmax, tau_pre=tau_pre, tau_post=tau_post,
                      dA_pre=dA_pre, dA_post=dA_post)
stdp2 = SongAbbottSTDP(C2, gmax=gmax, tau_pre=tau_pre, tau_post=tau_post,
                      dA_pre=dA_pre, dA_post=dA_post)


SI = SpikeMonitor(input)
S = SpikeMonitor(G)
tr_a = StateMonitor(G, 'vm', record=[2])
tr_T = StateMonitor(G, 'w', record=[2])
print "Before"
print C1.W
print C2.W


run(T)
subplot(211)
raster_plot(SI, S)
subplot(223)
plot(tr_a.times / ms, tr_a[2])
subplot(224)
plot(tr_T.times / ms, tr_T[2])
print "After"
print C1.W
print C2.W
show()



from brian import *

c1 = Clock(dt=2*ms, order=1)
c2 = Clock(dt=3*ms, order=2)
c3 = Clock(dt=5*ms, order=0)

@network_operation(clock=c1)
def f1():
    print '1:', c1.t
    
@network_operation(clock=c2)
def f2():
    print '2:', c2.t

@network_operation(clock=c3)
def f3():
    print '3:', c3.t

net = MagicNetwork()

net.run(20*ms)

print c1.t, c2.t, c3.t



from brian import *
set_global_preferences(usenewbrianhears=True)
from brian.hears import *

x = whitenoise(100*ms, nchannels=13)
x /= amax(abs(x))

hrtfset = ITDDatabase(13).load_subject()

fb = hrtfset.filterbank(Repeat(x, 2), interleaved=True)

y = fb.fetch(x.duration)
y += reshape(repeat(arange(13), 2), (1, 26))

plot(y)
show()




if criterion_name == 'LpError2':
    params['p'] = self.criterion.p
    params['varname'] = self.criterion.varname
    params['insets'] = self.criterion.insets
    params['outsets'] = self.criterion.outsets
    self.criterion_object = LpErrorCriterion2(**params)





class LpErrorCriterion2(Criterion):
    type = 'traces'
    def initialize(self, p=2, varname='v',insets=None,outsets=None):
        """
        Called at the beginning of every iteration. The keyword arguments here are
        specified in modelfitting.initialize_criterion().
        """
        self.p = double(p)
        self.varname = varname
        self._error = zeros((self.K, self.N))
        self.insets = insets
        self.outsets = outsets
        self.next_index=0
        self.insets = array(append(insets,inf),dtype=int)
        self.outsets =  array(append(outsets,inf),dtype=int)
        
    def timestep_call(self):
        v = self.get_value(self.varname)
        t = self.step()+1
        if t<self.onset: return # onset
        if t*self.dt >= self.duration: return
        if t>=self.insets[self.next_index]and t<=self.outsets[self.next_index]:
            if t==self.outsets[self.next_index]:
                self.next_index+=1
            d = self.intdelays
            indices = (t-d>=0)&(t-d<self.total_steps) # neurons with valid delays (stay inside the target trace)
            vtar = self.traces[:,t-d] # target value at this timestep for every neuron
            for i in xrange(self.K):
                self._error[i,indices] += abs(v[indices]-vtar[i,indices])**self.p
    
    def get_cuda_code(self):
        code = {}
        #    double p,
        #  DECLARATION
        code['%CRITERION_DECLARE%'] = """
    double *error_arr,
    double *insets,
    double *outsets,

        """
        
        # INITIALIZATION
        code['%CRITERION_INIT%'] = """
    double error = error_arr[neuron_index];
    int next_index=0;
        """
        
        # TIMESTEP
        #)
        code['%CRITERION_TIMESTEP%'] = """
        if ((T >= onset)&(Tdelay<duration-spikedelay-1)&(T >= insets[next_index])&(T <= outsets[next_index])) {
            if (T == outsets[next_index])
                {next_index+=1;}
            error = error + pow(abs(trace_value - %s), %.4f);
        }
        """ % (self.varname, self.p)
        
        # FINALIZATION
        code['%CRITERION_END%'] = """
    error_arr[neuron_index] = error;
        """
        
        return code
    
    def initialize_cuda_variables(self):
        """
        Initialize GPU variables to pass to the kernel
        """
        self.error_gpu = gpuarray.to_gpu(zeros(self.N, dtype=double))
        self.insets_gpu = gpuarray.to_gpu(self.insets)
        self.outsets_gpu = gpuarray.to_gpu(self.outsets)
    
    def get_kernel_arguments(self):
        """
        Return a list of objects to pass to the CUDA kernel.
        """
        args = [self.error_gpu,self.insets_gpu,self.outsets_gpu]
        return args
    
    def update_gpu_values(self):
        """
        Call gpuarray.get() on final values, so that get_values() returns updated values.
        """
        self._error = self.error_gpu.get()
        print self._error
    
    def get_values(self):
        if self.K == 1: error = self._error.flatten()
        else: error = self._error
        return error # just the integral, for every slice
    
    def normalize(self, error):
        # error is now the combined error on the whole duration (sum on the slices)
        # HACK: 1- because modelfitting MAXIMIZES for now...
#        self._norm = sum(self.traces**self.p, axis=1)**(1./self.p) # norm of every trace
        return 1-(self.dt*error)**(1./self.p)#/self._norm

class LpError2(CriterionStruct):
    """
    Structure used by the users to specify a criterion
    """
    def __init__(self, p = 2, varname = 'v',insets=None,outsets=None):
        self.type = 'trace'
        self.p = p
        self.insets = insets
        self.outsets = outsets
        self.varname = varname



from brian import *
from brian.hears import *
    
class RectifiedGammatone(CombinedFilterbank):
    def __init__(self, source, cf):
        CombinedFilterbank.__init__(self, source)
        source = self.get_modified_source()
        gfb = Gammatone(source, cf)
        output = FunctionFilterbank(gfb, lambda input: clip(input, 0, Inf))
        self.set_output(output)
        
x = whitenoise(100*ms)
fb = RectifiedGammatone(x, [1*kHz, 1.5*kHz])
y = fb.process()
subplot(211)
plot(y)
subplot(212)
fb.source = tone(1*kHz, 100*ms)
y = fb.process()
plot(y)
show()



from brian import *


class InputGenerator(object):
  """ A generator class to be passed to MultipleSpikeGeneratorGroup
  """
  def __init__(self, phase=0 * ms, period=1 * ms):
    self.phase = phase
    self.period = period
  def __call__(self):
    t = self.phase
    while(True):
      yield t
      t += self.period

ig1 = InputGenerator(3 * ms, 10 * ms)
ig2 = InputGenerator(5 * ms, 10 * ms)
G = MultipleSpikeGeneratorGroup([ig1, ig2])

print G._threshold.spiketimes
print G._threshold.spiketimeiter

M = SpikeMonitor(G)
net = Network(G, M)
net.run(20 * ms)
print M.spikes

net.reinit() # after reinit I don't see any firings

print defaultclock.t
print G._threshold.spiketimes
print G._threshold.spiketimeiter

ig1.phase = 5 * ms
ig2.phase = 5 * ms
net.run(20 * ms)
print M.spikes



from brian import *
from scipy.linalg import expm

def trapezoidal_integration(f, a, b, n):
    """
    Computes numerically the integral of the function f between a and b
    with a step dt, using the trapezoidal method.
    f(t) must accept vectors of t, and returns a D*n matrix, D is the dimension
    of the output space of f, n is the length of t.
    """
    I = f(a) + f(b)
    dt = (b - a) * 1.0 / n
    t = a + dt * arange(1, n)
    I += 2 * sum(f(t), axis=0)
    I *= dt / 2
    return I

def simpson_integration(f, a, b, n):
    """
    Computes numerically the integral of the function f between a and b
    with a step dt, using the Simpson method.
    f(t) must accept vectors of t, and returns a D*n matrix, D is the dimension
    of the output space of f, n is the length of t.
    """
    n = 2 * int(n / 2)
    I = f(a) + f(b)
    dt = (b - a) * 1.0 / n
    te = a + dt * arange(2, n, 2)
    to = a + dt * arange(1, n, 2)
    I += 2 * sum(f(te), axis=0)
    I += 4 * sum(f(to), axis=0)
    I *= dt / 3
    return I

def test_integration():
    a = randn(1)
    b = randn(1)
    c = randn(1)
    A = randn(1)
    B = A + randn(1) ** 2
    f = lambda x:a * exp(x) + b * x + c
    Itrap = trapezoidal_integration(f, A, B, 100)
    Isimp = simpson_integration(f, A, B, 100)
    Itrue = a * (exp(B) - exp(A)) + b * (B ** 2 / 2 - A ** 2 / 2) + c * (B - A)
    print abs(Itrap - Itrue), abs(Isimp - Itrue)

def compute_exact1(X0, b, t):
    a = array([[X0[0] + b[1], b[0] - X0[1]], [X0[1] - b[0], b[1] + X0[0]]])
    c = array([[cos(t)], [sin(t)]])
    b2 = array([[-b[1]], [b[0]]])
    X = dot(a, c) + b2
    return X.transpose()[0]

def compute_exact2(X0, b, t):
    X = array([(X0 + b) * exp(t) - b])
    return X

def compute_euler(X0, A, b, t, n):
    dt = t / n
    X = zeros((len(X0), n + 1))
    X[:, 0] = X0
    for i in range(n):
        X[:, i + 1] = X[:, i] + dt * (dot(A, X[:, i]) + b)
    return X

def compute_Ab(A, b, dt, nint):
    eAt = expm(A * dt)
    def f(u):
        if isscalar(u):
            return dot(expm(-A * u), b)
        else:
            return array([dot(expm(-A * v), b) for v in u])

    # numerical integration
    #b2 = dot(eAt, trapezoidal_integration(f, 0, dt, nint))
    
    # exact calculation when A invertible
    b2 = dot(dot(inv(A), eAt - eye(len(A))), b)

    return eAt, b2

def compute_exactstep(X0, A, b, t, n, nint):
    dt = t / n
    X = zeros((len(X0), n + 1))
    X[:, 0] = X0

#    eAt = expm(A*dt)
#    def f(u):
#        if isscalar(u):
#            return dot(expm(-A*u),b)
#        else:
#            return array([dot(expm(-A*v),b) for v in u])
#    eAtint = dot(eAt, trapezoidal_integration(f, 0, dt, nint))

    eAt, eAtint = compute_Ab(A, b, dt, nint)

    for i in range(n):
        X[:, i + 1] = dot(eAt, X[:, i]) + eAtint
    return X

def test_integral():
    a = 1000#randn(1)
    b = 1000#randn(1)
    dt = .0001
    nint = 10

    def f(u):
        if isscalar(u):
            return exp(-a * u) * b
        return reshape(exp(-a * u) * b, (len(u), 1))

    I = simpson_integration(f, 0, dt, nint)
    print "actual integration error       ", abs(I[0] - b / a * (1 - exp(-a * dt)))
    print "upper bound integration error  ", dt ** 5 / (180 * nint ** (4)) * exp(abs(a) * dt) * abs(b) * a ** 4

def test_sim():
    """
    Simulation of the system X' = AX+b, X(0) = X0, between 0 and T, with
    different integration schemes.
    """
    A = array([[0, -1], [1, 0]])
    D = len(A)
    b = 10 * randn(D)
    X0 = 10 * randn(D)

    T = 1.0
    dt = .0001
    n = 10000
    nint = 100

    X_exact = zeros((D, n + 1))
    for i in xrange(0, n + 1):
        X_exact[:, i] = compute_exact1(X0, b, i * dt)
    print "exact", X_exact[:, -1]

    X_euler = compute_euler(X0, A, b, T, n)
    print "euler error:      ", max(abs(X_euler[:,-1]-X_exact[:,-1]))

    X_exactstep = compute_exactstep(X0, A, b, T, n, nint)
    print "exact steps error:", max(abs(X_exactstep[:, -1] - X_exact[:, -1]))

def precision(dt, n, D, a, b):
    return dt ** 5 / (180 * n ** 4) * sqrt(D) * a ** 4 * exp(2 * a * dt) * b

def test_precision():
    dt = 1e-4
    n = 100
    D = 10
    a = b = 1000
    print precision(dt, n, D, a, b)

def is_equal(x, y, epsilon):
    return (abs(x - y) < abs(x) * epsilon).all()

if __name__ == '__main__':
    test_sim()
else:
    epsilon = 1.
    while 1. + epsilon > 1.:
        epsilon /= 2
    epsilon *= 2000.

    A = array([[0, -1], [1, 0]])
    D = len(A)
    b = 10 * randn(D)
    dt = .0001

    nint = 16
    btilde2 = compute_Ab(A, b, dt, nint)[1]
    for i in xrange(12):
        nint *= 2
        btilde = compute_Ab(A, b, dt, nint)[1]
        print nint, max(abs(btilde - btilde2)), max(abs(btilde)) * epsilon
        if max(abs(btilde - btilde2)) <= max(abs(btilde)) * epsilon: # or it's getting worse
            break
        btilde2 = btilde

        


from uuid import uuid4
from getpass import getuser
import os
import shelve
import platform
from glob import glob
from multiprocessing import Manager

__all__ = ['DataManager']


class LockingSession(object):
    def __init__(self, dataman, session_filename):
        self.dataman = dataman
        self.session_filename = session_filename
        self.lock = Manager().Lock()

    def acquire(self):
        self.lock.acquire()
        self.session = DataManager.shelf(self.session_filename)

    def release(self):
        self.session.close()
        self.session = None
        self.lock.release()

    def __getitem__(self, item):
        self.acquire()
        ret = self.session[item]
        self.release()
        return ret

    def __setitem__(self, item, value):
        self.acquire()
        self.session[item] = value
        self.release()


class DataManager(object):
    '''
    DataManager is a simple class for managing data produced by multiple
    runs of a simulation carried out in separate processes or machines. Each
    process is assigned a unique ID and Python Shelf object to write its data
    to. Each shelf is a dictionary whose keys must be strings. The DataManager
    can collate information across multiple shelves using the get(key) method,
    which returns a dictionary with keys the unique session names, and values
    the value written in that session (typically only the values will be of
    interest). If each value is a tuple or list then you can use the
    get_merged(key) to get a concatenated list. If the data type is more
    complicated you can use the get(key) method and merge by hand. The idea
    is each process generates files with names that do not interfere with each
    other so that there are no file concurrency issues, and then in the data
    analysis phase, the data generated separately by each process is merged
    together.
    
    Methods:
    
    ``get(key)``
        Return dictionary with keys the session names, and values the values
        stored in that session for the given key.
    ``get_merged(key)``
        Return a single list of the merged lists or tuples if each value for
        every session is a list or tuple.
    ``get_matching(match)``
        Returns a dictionary with keys the keys matching match and values
        get(key). If match is a string, a matching key has to start with that
        string. If match is a function, a key matches if match(key).
    ``get_merged_matching(match)``
        Like get_merged(key) but across all keys that match.
    ``get_flat_matching(match)``
        Returns a straight list of every value session[key] for all sessions
        and all keys matching match.
    ``iteritems()``
        Returns all ``(key, value)`` pairs, for each Shelf file, as an iterator
        (useful for large files with too much data to be loaded into memory).
    ``itervalues()``
        Return all values, for each Shelf file, as an iterator.
    ``items()``, ``values()``
        As for ``iteritems`` and ``itervalues`` but returns a list rather than
        an iterator.
    ``itemcount()``
        Returns the total number of items across all the Shelf files.
    ``keys()``
        A list of all the keys across all sessions.
    ``session()``
        Returns a randomly named session Shelf, multiple processes can write to
        these without worrying about concurrency issues.
    ``computer_session()``
        Returns a consistently named Shelf specific to that user and computer,
        only one process can write to it without worrying about concurrency issues.
    ``locking_session()``, ``locking_computer_session()``
        Returns a LockingSession object, a limited proxy to the underlying
        Shelf which acquires and releases a lock before and after every
        operation, making it safe for concurrent access.
    ``session_filenames()``
        A list of all the shelf filenames for all sessions.
    ``make_unique_key()``
        Generates a unique key for inserting an element into a session without
        overwriting data, uses uuid4.
        
    Attributes:
    
    ``basepath``
        The base path for data files.
    ``computer_name``
        A (hopefully) unique identifier for the user and computer, consists of
        the username and the computer network name.
    ``computer_session_filename``
        The filename of the computer-specific session file. This file should
        only be accessed by one process at a time, there's no way to protect
        against concurrent write accesses causing it to be corrupted.
    '''
    def __init__(self, name, datapath=''):
        # check if directory exists, and if not, make it
        basepath = os.path.join(datapath, name + '.data')
        if not os.path.exists(basepath):
            subpaths = (name + '.data').split('/')
            curpath = ''
            for path in subpaths:
                curpath += path
                if not os.path.exists(os.path.join(datapath, curpath)):
                    os.mkdir(os.path.join(datapath, curpath))
                curpath += '/'
        self.basepath = basepath
        self.computer_name = getuser() + '.' + platform.node()
        self.computer_session_filename = self.session_filename(self.computer_name)

    def session_name(self):
        return getuser() + '.' + str(uuid4())

    def session_filename(self, session_name=None):
        if session_name is None:
            session_name = self.session_name()
        fname = os.path.normpath(os.path.join(self.basepath, session_name))
        return fname
    @staticmethod
    def shelf(fname):
        return shelve.open(fname, protocol=2)

    def session(self, session_name=None):
        return self.shelf(self.session_filename(session_name))

    def computer_session(self):
        return self.shelf(self.computer_session_filename)

    def locking_session(self, session_name=None):
        return LockingSession(self, self.session_filename(session_name))

    def locking_computer_session(self):
        return LockingSession(self, self.computer_session_filename)

    def session_filenames(self):
        return glob(os.path.join(self.basepath, '*'))

    def get(self, key):
        allfiles = self.session_filenames()
        ret = {}
        for name in allfiles:
            path, file = os.path.split(name)
            shelf = shelve.open(name, protocol=2)
            if key in shelf:
                ret[file] = shelf[key]
        return ret

    def get_merged(self, key):
        allitems = self.get(key)
        ret = []
        for _, val in allitems.iteritems():
            if isinstance(val, (list, tuple)):
                ret.extend(val)
            else:
                raise TypeError('Can only get merged items of list or tuple type, use get() method and merge by hand.')
        return ret

    def get_matching_keys(self, match):
        allkeys = self.keys()
        matching_keys = [key for key in allkeys if (callable(match) and match(key)) or key.startswith(match)]
        return matching_keys

    def get_matching(self, match):
        ret = {}
        for key in self.get_matching_keys(match):
            ret[key] = self.get(key)
        return ret

    def get_merged_matching(self, match):
        ret = []
        for key in self.get_matching_keys(match):
            ret.extend(self.get_merged(key))
        return ret

    def get_flat_matching(self, match):
        allitems = self.get_matching(match)
        ret = []
        for matching_key, matching_dict in allitems.iteritems():
            ret.extend(matching_dict.values())
        return ret

    def keys(self):
        allkeys = set([])
        for name in self.session_filenames():
            allkeys.update(set(shelve.open(name, protocol=2).keys()))
        return list(allkeys)
    
    def make_unique_key(self):
        return str(uuid4())
    
    def iteritems(self):
        allfiles = self.session_filenames()
        for name in allfiles:
            shelf = shelve.open(name, protocol=2)
            for key, value in shelf.iteritems():
                yield key, value
                
    def itervalues(self):
        for key, val in self.iteritems():
            yield val
    
    def items(self):
        return list(self.iteritems())
            
    def values(self):
        return list(self.itervalues())
    
    def itemcount(self):
        return sum(len(shelve.open(name, protocol=2)) for name in self.session_filenames())

if __name__ == '__main__':
    d = DataManager('test/testing')
    #s = d.session()
    #s['a'] = [7]
    print d.get_merged('a')

    #d = DataManager('test')
    #print d.get_merged('b')
    #print d.keys()
    #print d['b']
    #s = d.session()
    #s['b'] = 6



from numpy import zeros
import numpy
from pycuda import autoinit
from pycuda import gpuarray
from pycuda import compiler
x = gpuarray.to_gpu(zeros(10, numpy.float32))
src = '''
__global__ void f(float *x)
{
 x[0] = exp(x[0]);
}
'''
gpu_mod = compiler.SourceModule(src)
gpu_func = gpu_mod.get_function("f")
gpu_func(x, block=(1, 1, 1), grid=(1, 1))
print x.get()



from numpy import *
from multiprocessing.connection import Client

N = 10000
numprocesses = 6
complexity = 10

address = ('localhost', 2718)
conn = Client(address, authkey='secret password')
print 'Connection acquired'

x = ones(N)

conn.send(x)
print 'Shared data sent'

results = []
for a in xrange(numprocesses):
    conn.send((a, complexity))
    results.append(conn.recv())

print results

conn.send(None)

conn.close()



"""
Electrode compensation
----------------------
Two methods are implemented:
* Lp parametric compensation
* Active Electrode Compensation (AEC)
"""
from brian.stateupdater import get_linear_equations
from brian.log import log_info
from brian import second, Mohm, mV, ms, Equations, ohm, volt, second
from scipy.optimize import fmin
from scipy.signal import lfilter
from scipy import linalg
from numpy import sqrt, ceil, zeros, eye, poly, dot, hstack, array
from scipy import zeros, array, optimize, mean, arange, diff, rand, exp, sum, convolve, eye, linalg, sqrt
import time

__all__=['Lp_compensate', 'full_kernel', 'full_kernel_from_step',
         'electrode_kernel_soma', 'electrode_kernel_dendrite', 'solve_convolution',
         'electrode_kernel', 'AEC_compensate']

def compute_filter(A, row=0):
    """
    From a linear discrete-time multidimensional system Y(n+1)=AY(n)+X(n),
    with X(n) a vector with X(n)[1:]==0,
    compute a linear 1D filter for simulating the variable indexed by "row".
    It returns two vectors b, a, that are to be provided to the Scipy
    function lfilter.
    """
    d = len(A)

    # compute a: characteristic polynomial of the matrix
    a = poly(A)  # with a[0]=1

    # compute b recursively
    b = zeros(d+1)
    T = eye(d)
    b[0] = T[row, 0]
    for i in range(1, d+1):
        T = a[i]*eye(d) + dot(A, T)
        b[i] = T[row, 0]

    return b, a

def simulate(eqs, I, dt, row=0): # export?
    """
    Simulate a system of neuron equations in response to an injected current I
    with sampling frequency 1/dt, using a Scipy linear filter rather than a Brian
    integrator (it's more efficient since there are no spikes). 
    The variable indexed by "row" only is simulated.
    I must not appear in the equations, rather, it will be injected in the 
    *first* variable only. That is to say, if the equations correspond to the
    differential system dY/dt=MY+I, where only the first component of I is nonzero,
    then the equations must correspond to the matrix M. 
    For instance, if the equations were dv/dt=(R*I-v)/tau, then the following
    equation must be given: "dv/dt=-v/tau", and the input current will be
    "I*R/tau".
    """
    # get M and B such that the system is dY/dy=M(Y-B)
    # NOTE: it only works if the system is invertible (det(M)!=0) !
    M, B = get_linear_equations(eqs)
    # discretization of the system: Y(n+1)=AY(n), were A=exp(M*dt).
    A = linalg.expm(M * dt)
    # compute the filter
    b, a = compute_filter(A, row=row)
    # apply the filter to the injected current
    y = lfilter(b, a, I*dt) + B[row]
    return y

"""
Lp Electrode Compensation
-------------------------
From: Rossant et al (?): to be submitted.
"""
class ElectrodeCompensation (object):
    """
    Lp Electrode Compensation class.
    ================================

    Implements an electrode compensation technique based on linear model 
    fitting of an electrode and a neuron in response to an injected current.
    The fitness function is the Lp error between the full model response
    and the raw trace, with p<2 to minimize the bias due to the nonlinear
    voltage excursions of action potentials.
    The fitting procedure uses the Scipy fmin optimization function.
    The model simulation is performed efficiently on the CPU using a
    linear filter directly applied to the current. This filter is
    computed dynamically from the electrode and neuron differential equations.
    """
    eqs = """
            dV/dt=Re*(-Iinj)/taue : volt
            dV0/dt=(R*Iinj-V0+Vr)/tau : volt
            Iinj=(V-V0)/Re : amp
            """

    def __init__(self, I, Vraw,
                 dt, durslice=10*second,
                  p=1.0, 
                 *params):
        """
        Class constructor.

        * I: injected current, 1D vector.
        * Vraw: raw (uncompensated) voltage trace, 1D vector, same length as I.
        * dt: sampling period (inverse of the sampling frequency), in second.
        * durslice=1*second: duration of slices: the fit is performed independently on each slice
          to capture the possible parameter changes during the recordings.
        * p=1.0: parameter of the Lp error. In general, p<2, and a 
          smaller value for p (like 0.5) will yield better results, especially if
          there are a lot of action potentials.
        * *params: a list of initial parameters for the optimization, in the following
          order: R, tau, Vr, Re, taue.
        """
        self.I = I
        self.Vraw = Vraw
        self.p = p
        self.dt = dt
        self.dt_ = float(dt)
        self.x0 = self.params_to_vector(*params)
        self.duration = len(I) * dt
        self.durslice = min(durslice, self.duration)
        self.slicesteps = int(durslice/dt)
        self.nslices = int(ceil(len(I)*dt/durslice))
        
        self.islice = 0
        self.I_list = [I[self.slicesteps*i:self.slicesteps*(i+1)] for i in range(self.nslices)]
        self.Vraw_list = [Vraw[self.slicesteps*i:self.slicesteps*(i+1)] for i in range(self.nslices)]

    def vector_to_params(self, *x):
        """
        Convert a vector of parameters (used for the optimization) to a tuple
        of actual parameters.
        """
        R,tau,Vr,Re,taue = x

        R = R*R
        tau = tau*tau
        Re = Re*Re
        taue = taue*taue

        return R,tau,Vr,Re,taue

    def params_to_vector(self, *params):
        """
        Opposite of vector_to_params.
        """
        x = params

        x = [sqrt(params[0]),
             sqrt(params[1]),
             params[2],
             sqrt(params[3]),
             sqrt(params[4])]

        return list(x)

    def get_model_trace(self, row, *x):
        """
        Compute the model response (variable index "row") to the injected current,
        at a specific slice (stored in self.islice), with model parameters
        specified with the vector x.
        """
        R, tau, Vr, Re, taue = self.vector_to_params(*x)
        # put units again
        R, tau, Vr, Re, taue = R*ohm, tau*second, Vr*volt, Re*ohm, taue*second
        eqs = Equations(self.eqs)
        eqs.prepare()
        self._eqs = eqs
        y = simulate(eqs, self.I_list[self.islice] * Re/taue, self.dt, row=row)
        return y

    def fitness(self, x):
        """
        fitness function provided to the fmin optimization procedure.
        Simulate the model and compute the Lp error between the model
        response and the raw trace.
        """
        R, tau, Vr,  Re, taue = self.vector_to_params(*x)
        y = self.get_model_trace(0, *x)
        e = self.dt_*sum(abs(self.Vraw_list[self.islice]-y)**self.p)
        return e

    def compensate_slice(self, x0):
        """
        Compensate on the current slice, by calling fmin on the fitness function.
        """
        fun = lambda x: self.fitness(x)
        x = fmin(fun, x0, maxiter=10000, maxfun=10000, disp=False)
        return x

    def compensate(self):
        """
        Compute compensate_slice for all slices.
        Use the previous best parameters as initial parameters for the next slice.
        """
        self.params_list = []
        self.xlist = [self.x0]
        t0 = time.clock()
        for self.islice in range(self.nslices):
            newx = self.compensate_slice(self.xlist[self.islice])
            self.xlist.append(newx)
            self.params_list.append(self.vector_to_params(*newx))
            log_info("electrode_compensation","Slice %d/%d compensated in %.2f seconds" %  \
                (self.islice+1, self.nslices, time.clock()-t0))
            t0 = time.clock()
        self.xlist = self.xlist[1:]
        return self.xlist

    def get_compensated_trace(self):
        """
        Called after the compensation procedure. Compute the full model
        traces, for all slices, using the best parameters found by the
        optimization. Returns only the compensated trace, but this
        function also computes the neuron and electrode responses.
        """
        Vcomp_list = []
        Vneuron_list = []
        Velec_list = []
        
        for self.islice in range(self.nslices):
            x = self.xlist[self.islice]
            V = self.get_model_trace(0, *x)
            V0 = self.get_model_trace(1, *x)
            Velec = V-V0
            
            Vneuron_list.append(V0)
            Velec_list.append(Velec)
            Vcomp_list.append(self.Vraw_list[self.islice] - Velec)
            
        self.Vcomp = hstack(Vcomp_list)
        self.Vneuron = hstack(Vneuron_list)
        self.Velec = hstack(Velec_list)
        
        return self.Vcomp

def Lp_compensate(I, Vraw, dt, 
               slice_duration=1*second,
               p=1.0,
               full=False,
               **initial_params):
    """
    Lp Electrode Compensation.

    Implements an electrode compensation technique based on linear model 
    fitting of an electrode and a neuron in response to an injected current.
    The fitness function is the Lp error between the full model response
    and the raw trace, with p<2 to minimize the bias due to the nonlinear
    voltage excursions of action potentials.
    The fitting procedure uses the Scipy fmin optimization function.
    The model simulation is performed efficiently on the CPU using a
    linear filter directly applied to the current. This filter is
    computed dynamically from the electrode and neuron differential equations.

    Arguments
    * I: injected current, 1D vector.
    * Vraw: raw (uncompensated) voltage trace, 1D vector, same length as I.
    * dt: sampling period (inverse of the sampling frequency), in second.
    * slice_duration=1*second: duration of slices: the fit is performed independently on each slice
        to capture the possible parameter changes during the recordings.
    * p=1.0: parameter of the Lp error. In general, p<2, and a 
        smaller value for p (like 0.5) will yield better results, especially if
        there are a lot of action potentials.
    * **initial_params: initial parameters for the optimization: R, tau, Vr, Re, taue.
        * R: neuron resistance, default 100 MOhm
        * tau: neuron membrane time constant, default 20 ms
        * Vr: neuron rest potentiel, default -70 mV
        * Re: electrode resistance, default 50 MOhm
        * taue: electrode time constant, default 0.5 ms
    """
    R = initial_params.get("R", 100*Mohm)
    tau = initial_params.get("tau", 20*ms)
    Vr = initial_params.get("Vr", -70*mV)
    Re = initial_params.get("Re", 50*Mohm)
    taue = initial_params.get("taue", .5*ms)

    comp = ElectrodeCompensation(I, Vraw,
                                 dt,
                                 slice_duration,
                                 p,
                                 R, tau, Vr, Re, taue)
    comp.compensate()
    Vcomp = comp.get_compensated_trace()
    params = array(comp.params_list).transpose()
    if not full:
        return Vcomp, params
    else:
        return dict(Vcompensated=Vcomp, Vneuron=comp.Vneuron,
                    Velectrode=comp.Velec, params=params)

'''
Active Electrode Compensation
-----------------------------
From:
Brette et al (2008). High-resolution intracellular recordings using a real-time
computational model of the electrode. Neuron 59(3):379-91.
'''
def full_kernel(v, i, ksize, full_output=False):
    '''
    Calculates the full kernel from the recording v and the input
    current i. The last ksize steps of v should be null.
    ksize = size of the resulting kernel
    full_output = returns K,v0 if True (v0 is the resting potential)
    '''
    # Calculate the correlation vector <v(n)i(n-k)>
    # and the autocorrelation vector <i(n)i(n-k)>
    vi = zeros(ksize)
    ii = zeros(ksize)
    vref = mean(v) # taking <v> as the reference potential simplifies the formulas
    for k in range(ksize):
        vi[k] = mean((v[k:] - vref) * i[:len(i) - k])
        ii[k] = mean(i[k:] * i[:len(i) - k])
    vi -= mean(i) ** 2
    K = levinson_durbin(ii, vi)
    if full_output:
        v0 = vref - mean(i) * sum(K)
        return K, v0
    else:
        return K

def full_kernel_from_step(V, I):
    '''
    Calculates the full kernel from the response (V) to a step input
    (I, constant).
    '''
    return diff(V) / I

def solve_convolution(K, Km):
    '''
    Solves Ke = K - Km * Ke/Re
    Linear problem
    '''
    Re = sum(K) - sum(Km)
    n = len(Km)
    A = eye(n) * (1 + Km[0] / Re)
    for k in range(n):
        for m in range(k):
            A[k, m] = Km[k - m] / Re
    return linalg.lstsq(A, K)[0]

def electrode_kernel_dendrite(Karg, start_tail, full_output=False):
    '''
    (For dendritic recordings)
    Extracts the electrode kernel Ke from the raw kernel K
    by removing the membrane kernel, estimated from the
    indexes >= start_tail of the raw kernel.
    full_output = returns Ke,Km if True (otherwise Ke)
    (Ke=electrode filter, Km=membrane filter)
    '''

    K = Karg.copy()

    def remove_km(RawK, Km):
        '''
        Solves Ke = RawK - Km * Ke/Re for a dendritic Km.
        '''
        Kel = RawK - Km
        # DOES NOT CONVERGE!!
        for _ in range(5): # Iterative solution
            Kel = RawK - convolve(Km, Kel)[:len(Km)] / sum(Kel)
            # NB: Re=sum(Kel) increases after every iteration
        return Kel

    # Fit of the tail to a dendritic kernel to find the membrane time constant
    t = arange(len(K))
    tail = arange(start_tail, len(K))
    Ktail = K[tail]
    f = lambda params:params[0] * ((tail + 1) ** -.5) * exp(-params[1] ** 2 * (tail + 1)) - Ktail
    #Rtail=sum(Ktail)
    #g=lambda tau:sum((tail+1)**(-.5)*exp(-(tail+1)/tau))
    #J=lambda tau:sum(((tail+1)**(-.5)*exp(-(tail+1)/tau)/g(tau)-Ktail/Rtail)**2)
    p, _ = optimize.leastsq(f, array([1., .03]))
    #p=optimize.fminbound(J,.1,10000.)
    #p=optimize.golden(J)

    #print "tau_dend=",p*.1
    #Km=(t+1)**(-.5)*exp(-(t+1)/p)*Rtail/g(p)

    print "tau_dend=", .1 / (p[1] ** 2)
    Km = p[0] * ((t + 1) ** -.5) * exp(-p[1] ** 2 * (t + 1))
    K[tail] = Km[tail]

    # Find the minimum
    z = optimize.fminbound(lambda x:sum(solve_convolution(K, x * Km)[tail] ** 2), .5, 1.)
    Ke = solve_convolution(K, z * Km)

    if full_output:
        return Ke[:start_tail], z * Km
    else:
        return Ke[:start_tail]

def electrode_kernel_soma(Karg, start_tail, full_output=False):
    '''
    (For somatic recordings - alternative method)
    Extracts the electrode kernel Ke from the raw kernel K
    by removing the membrane kernel, estimated from the
    indexes >= start_tail of the raw kernel.
    full_output = returns Ke,Km if True (otherwise Ke)
    (Ke=electrode filter, Km=membrane filter)
    '''

    K = Karg.copy()

    def remove_km(RawK, Km):
        '''
        Solves Ke = RawK - Km * Ke/Re for a dendritic Km.
        '''
        Kel = RawK - Km
        for _ in range(5): # Iterative solution
            Kel = RawK - convolve(Km, Kel)[:len(Km)] / sum(Kel)
            # NB: Re=sum(Kel) increases after every iteration
        return Kel

    # Fit of the tail to a somatic kernel to find the membrane time constant
    t = arange(len(K))
    tail = arange(start_tail, len(K))
    Ktail = K[tail]
    f = lambda params:params[0] * exp(-params[1] ** 2 * (tail + 1)) - Ktail
    p, _ = optimize.leastsq(f, array([1., .3]))
    Km = p[0] * exp(-p[1] ** 2 * (t + 1))
    print "tau_soma=", .1 / (p[1] ** 2)

    K[tail] = Km[tail]

    # Find the minimum
    z = optimize.fminbound(lambda x:sum(solve_convolution(K, x * Km)[tail] ** 2), .5, 1.)
    Ke = solve_convolution(K, z * Km)
    print "R=", sum(z * p[0] * exp(-p[1] ** 2 * (arange(1000) + 1)))

    if full_output:
        return Ke[:start_tail], z * Km
    else:
        return Ke[:start_tail]

def electrode_kernel(Karg, start_tail, full_output=False):
    '''
    Extracts the electrode kernel Ke from the raw kernel K
    by removing the membrane kernel, estimated from the
    indexes >= start_tail of the raw kernel.
    full_output = returns Ke,Km if True (otherwise Ke)
    (Ke=electrode filter, Km=membrane filter)
    
    Finds automatically whether to use dendritic or somatic kernel.
    '''

    K = Karg.copy()

    # Fit of the tail to a somatic kernel to find the membrane time constant
    t = arange(len(K))
    tail = arange(start_tail, len(K))
    Ktail = K[tail]
    f = lambda params:params[0] * exp(-params[1] ** 2 * (tail + 1)) - Ktail
    p, _ = optimize.leastsq(f, array([1., .3]))
    Km_soma = p[0] * exp(-p[1] ** 2 * (t + 1))

    f = lambda params:params[0] * ((tail + 1) ** -.5) * exp(-params[1] ** 2 * (tail + 1)) - Ktail
    p, _ = optimize.leastsq(f, array([1., .03]))
    Km_dend = p[0] * ((t + 1) ** -.5) * exp(-p[1] ** 2 * (t + 1))

    if sum((Km_soma[tail] - Ktail) ** 2) < sum((Km_dend[tail] - Ktail) ** 2):
        print "Somatic kernel"
        Km = Km_soma
    else:
        print "Dendritic kernel"
        Km = Km_dend

    K[tail] = Km[tail]

    # Find the minimum
    z = optimize.fminbound(lambda x:sum(solve_convolution(K, x * Km)[tail] ** 2), .5, 1.)
    Ke = solve_convolution(K, z * Km)

    if full_output:
        return Ke[:start_tail], z * Km
    else:
        return Ke[:start_tail]

def AEC_compensate(v, i, ke):
    '''
    Active Electrode Compensation, done offline.
    v = recorded potential
    i = injected current
    ke = electrode kernel
    Returns the compensated potential.
    '''
    return v - convolve(ke, i)[:-(len(ke) - 1)]

def levinson_durbin(a, y):
    '''
    Solves AX=Y where A is a symetrical Toeplitz matrix with coefficients
    given by the vector a (a = first row = first column of A).
    '''
    b = 0 * a
    x = 0 * a
    b[0] = 1. / a[0]
    x[0] = y[0] * b[0]
    for i in range(1, len(a)):
        alpha = sum(a[1:i + 1] * b[:i])
        u = 1. / (1 - alpha ** 2)
        v = -alpha * u
        tmp = b[i - 1]
        if i > 1:
            b[1:i] = v * b[i - 2::-1] + u * b[:i - 1]
        b[0] = v * tmp
        b[i] = u * tmp
        beta = y[i] - sum(a[i:0:-1] * x[:i])
        x += beta * b
    return x



'''
Distributed simulations on clusters, using MPI.

Groups hosted on other machines are mapped to virtual groups.
Connection objects do not change (simply, source or target may be virtual).
The ClientNetwork object runs the network normally, except it sends and receives spikes
from times to times to other machines.
The ServerNetwork object creates ClientNetwork objects and distributes them over the
machines. It does not need supervise the simulation.

How about creation? (in particular, connection objects should be created on the target machine)
    It should be possible to create the objects on the target machines, e.g.:
    * create the group/connection on the server
    * communicate the object to the client
    * convert to virtual group on the server

How about monitoring?
    Simply communicate the monitors to the machine hosting the relevant neuron group.

How about plasticity?

Objects on a node:
* (real) groups
* virtual groups
* connections to groups only (from groups and virtual groups)

A time step involves:
* normal calls to real groups (update, threshold, reset)
* broadcast of spikes from all groups; can be done in the threshold operation
* usual connection propagate

Essentially, what needs to be done is:
* removing update/reset from virtual groups
* having a special threshold in virtual group which simply does a broadcast
* adding the broadcast to real groups

Note that this mechanism can handle homogeneous delays in a simple way
(the LS structure pushes the spikes as usual).
Thus, the main difficulty is setting up the simulation.
There can also difficulties with monitors and network operations.

How to distribute the groups?
1) Simplest possibility: use preexisting groups, do not split them.
Then simply divide them equally among processors.
2) Divide every group equally in the number of processors.
3) Divide the set of groups between processors, then divide each group.

Strategy 2 has an advantage: NeuronGroups and Connections can be divided
at initialisation time. Then the connect functions of Connection could be
overriden so that the matrices are only created on the clients
(faster and much better for memory). It could be done on the same way
for neuron groups, but it is probably sufficient to do the construction on
the server and do the transfer when the network run is called. However with
network operations, state transfers should be possible at run time.

One simple possibility would be something like:
from brian import *
from brian.cluster import *

and the latter statement would redefine NeuronGroup, Connection, etc.
Something like:
NeuronGroup = ClusterNeuronGroup
Connection = ClusterConnection

The same script could be run on all processors, that could make
things much simpler. For example, consider 2 processors and the statement
P=NeuronGroup(100,model=...)
creates a ClusterNeuronGroup on each processor, in which either
P[:50] or P[50:] is real while the rest is virtual (it may not be
necessary).
Then P.v=rand(100) would only assign the relevant part of the vector.
It seems to be wasting a lot of time since the same operation is repeated on
all processors, but they are done in parallel so it is not less efficient than
doing everything on a server and transferring the data (in fact it is more
efficient since we avoid the transfers).
Connections would work in the same way, with minor code changes.
Monitors could also work similarly, which would work as
if each group P were replaced by a subgroup.

The only asymmetry would be that processor 0 would gather all information
from monitors at the end, and possibly some analysis, plotting, etc.
So there should probably be some specific code for the server.
That makes it less interesting to use the same script for all nodes...

Also, the statement
from brian.cluster import *
could be avoided by placing a test in brian.__init__ that checks the number
of processor (i.e., whether the script has been run with mpiexec).
Alternatively, one could use a special Python script to run a script on
a cluster (which would call mpiexec).

ServerGroup:
* stores all state variables normally
* transfers only when network is running (e.g. use a flag when running)
* transfers are triggered by reads/writes at run time 
(gather operation?); more specifically:
   + write operation triggers a flag for transfer at end of time step
   + read operation triggers an immediate transfer if flag is up and pushes
   flag down (flag up at end of time step).
* updates, threshold, resets only concern the beginning of the group
(non virtual)
A NeuronGroup could be turned into a ServerGroup when the first
run function is called. This way all subclasses of NeuronGroup could be
used. The ServerGroup holds a reference to the real NeuronGroup and
inserts the required operations.
This way, monitors can stay on the server side, essentially unchanged.
The problem with this strategy is that plasticity is implemented with
monitors.

In fact on the client side, one could use the same class as ServerGroup,
except the indexes of the real subgroup are different. Reads/writes would
be forbidden in virtual groups (not a problem if monitors stay on the
server side).

What might be necessary for transparent use is that all objects (groups,
connections) should have a subgrouping mechanism.

Yet another strategy based on option 1), i.e., do not subdivide groups:
* When a NeuronGroup is created, it is decided internally whether it should
be on the main server or on a client (dynamic load balancing based only on
the number of neurons). The corresponding groups are sent to the clients.
* When a Connection is created, it already knows where the groups are, so
it is simple to handle.
* That technique duplicates all spike monitors, but maybe we can change that
behaviour.
'''
import pypar
from brian.neurongroup import *
from brian.network import *
from brian.globalprefs import set_global_preferences
from cluster_client import run_client


class VirtualGroup(NeuronGroup):
    '''
    A group that is hosted on another machine.
    A number of methods will raise an exception (e.g. accessing the state variables).
    '''
    def __init__(self, group, machine):
        '''
        Initializes the virtual group:
            group: real group
            machine: machine id
        '''
        # How do we deal with clocks?
        self._S0 = group._S0
        self.staticvars = group.staticvars
        self.var_index = group.var_index
        self._max_delay = group._max_delay
        self.LS = group.LS
        self._owner = machine
        self._length = len(group)
        self._numstates = group.num_states()

    # Update and reset are disabled
    def update(self):
        pass
    def reset(self):
        pass
    def __len__(self):
        return self._length
    def num_states(self):
        return self._numstates
    def __repr__(self):
        return 'Virtual group of ' + str(len(self)) + ' neurons'


class ServerNetwork(Network):
    '''
    Network class for running a simulation over a cluster.
    The server manages the clients.
    '''
    pass

# Identification
myid = pypar.rank() # id of this process
nproc = pypar.size() # number of processors

if myid > 0: # client
    import sys
    run_client()
    sys.exit(0)

# Server
#Network=ServerNetwork
set_global_preferences(cluster_server=True)



from brian import *
import brianlib as bl

G = NeuronGroup(5, 'v:1')
H = NeuronGroup(3, 'v:1')
C = Connection(G, H, structure='dense')
C.W[:] = randn(*C.W.shape)

blG = bl.NeuronGroup(G._S, None, None, None, G.LS.S.n, G.LS.ind.n)
blH = bl.NeuronGroup(H._S, None, None, None, H.LS.S.n, H.LS.ind.n)
blC_W = bl.DenseConnectionMatrix(C.W)
blC = bl.Connection(blG, blH, blC_W)
x = arange(5)
#blC_W.add_row(0, H._S[0])
#blC_W.add_rows(x, H._S[0])
blC.propagate(x)

print C.W
print H._S
print sum(C.W, axis=0)



from brian import *

eqs = '''
dV/dt = (0.5-V)/(10*ms) : 1
refrac : second
'''

#G = NeuronGroup(10, eqs, reset=0, threshold=1, refractory=0.5*ms)
#G = NeuronGroup(10, eqs, reset=0, threshold=1, refractory=linspace(0*ms, 5*ms, 10, endpoint=False))
G = NeuronGroup(10, eqs, reset=0, threshold=1, refractory='refrac', max_refractory=10 * ms)
G.refrac = linspace(0 * ms, 5 * ms, 10, endpoint=False)
G.V = 2

M = StateMonitor(G, 'V', record=True)

run(1 * ms)

#print G._next_allowed_spiketime 

run(9 * ms)

G.V = 2
#G.refrac = linspace(2*ms, 0*ms, 10, endpoint=False)

run(5 * ms)

M.plot()
show()
